@@ -1,6 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: LMfit.h 503 2009-06-09 15:47:31Z wojdyr $
 
 #ifndef FITYK__LMFIT__H__
 #define FITYK__LMFIT__H__
@@ -16,16 +15,14 @@ class LMfit : public Fit
 public:
     LMfit(Ftk* F);
     ~LMfit();
-    fp init(); // called before do_iteration()/autoiter()
+    virtual void init(); // called before do_iteration()/autoiter()
     void autoiter();
 private:
-    fp shake_before;
-    char shake_type;
-    std::vector<fp> alpha, alpha_;            // matrices
-    std::vector<fp> beta, beta_;   // and vectors
-    std::vector<fp> a;    // parameters table
-    fp chi2 , chi2_;
-    fp lambda;
+    std::vector<realt> alpha, alpha_;            // matrices
+    std::vector<realt> beta, beta_;   // and vectors
+    std::vector<realt> a;    // parameters table
+    realt chi2, chi2_;
+    double lambda;
 
     bool do_iteration();
 };
