@@ -1,6 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: NMfit.h 503 2009-06-09 15:47:31Z wojdyr $
 
 #ifndef FITYK__NMFIT__H__
 #define FITYK__NMFIT__H__
@@ -12,13 +11,13 @@
 /// Vertex used in Nelder-Mead method
 struct Vertex
 {
-    std::vector<fp> a;
+    std::vector<realt> a;
     bool computed;
-    fp wssr;
+    realt wssr;
 
-    Vertex() : a(0), computed(false) {}
-    Vertex(int n) : a(n), computed(false) {}
-    Vertex (std::vector<fp>& a_) : a(a_), computed(false) {}
+    Vertex() : a(0), computed(false), wssr(0.) {}
+    Vertex(int n) : a(n), computed(false), wssr(0.) {}
+    Vertex(std::vector<realt>& a_) : a(a_), computed(false), wssr(0.) {}
 };
 
 ///              Nelder-Mead simplex method
@@ -27,21 +26,21 @@ class NMfit : public Fit
 public:
     NMfit(Ftk* F);
     ~NMfit();
-    fp init(); // called before autoiter()
+    virtual void init(); // called before autoiter()
     void autoiter();
 private:
     int iteration;
     std::vector<Vertex> vertices;
     std::vector<Vertex>::iterator best, s_worst /*second worst*/, worst;
-    std::vector<fp> coord_sum;
-    fp volume_factor;
+    std::vector<realt> coord_sum;
+    realt volume_factor;
 
     void find_best_worst();
     void change_simplex();
-    fp try_new_worst (fp f);
+    realt try_new_worst(realt f);
     void compute_coord_sum();
-    bool termination_criteria (int iter, fp convergence);
-    void compute_v (Vertex& v);
+    bool termination_criteria(int iter, realt convergence);
+    void compute_v(Vertex& v);
 };
 
 #endif
