@@ -1,23 +1,9 @@
 // This file is part of fityk program. Copyright (C) 2005 Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: ast.cpp 748 2010-06-14 17:44:34Z wojdyr $
 
-//  Based on ast_calc example from Boost::Spirit by Daniel Nuffer
-
-//TODO:
-// CSE in tree (or in VM code?)
-// new op: SQR? DUP? STORE/WRITE?
-// AST -> VM code
-// output VM code for tests
-// constant-merge (merge identical constants)
-
-
-#include <boost/spirit/include/classic_core.hpp>
-#include <boost/spirit/include/classic_ast.hpp>
 #include <boost/math/special_functions/gamma.hpp>
 #include <boost/math/special_functions/digamma.hpp>
 
-#include <sstream>
 #include <string>
 #include <vector>
 #include <cassert>
@@ -28,29 +14,14 @@
 #include "ui.h"
 #include "ast.h"
 #include "var.h"
-#include "datatrans.h"
 #include "logic.h"
 #include "numfuncs.h"
 #include "voigt.h"
+#include "lexer.h"
+#include "eparser.h"
 
-////////////////////////////////////////////////////////////////////////////
 using namespace std;
-using namespace boost::spirit::classic;
 
-//typedef char const*         iterator_t;
-//typedef tree_match<iterator_t> parse_tree_match_t;
-//typedef parse_tree_match_t::tree_iterator iter_t;
-//typedef parse_tree_match_t::const_tree_iterator const_iter_t;
-//typedef tree_match<char const*>::const_tree_iterator const_iter_t;
-//typedef tree_match<char const*>::const_tree_iterator const_tm_iter_t;
-
-////////////////////////////////////////////////////////////////////////////
-
-
-OpTree::OpTree(int n, OpTree *arg1) : op(n), c1(arg1), c2(0), val(0.)
-                              { assert(n >= OP_ONE_ARG && n < OP_TWO_ARG); }
-OpTree::OpTree(int n, OpTree *arg1, OpTree *arg2)
-    : op(n), c1(arg1), c2(arg2), val(0.)   { assert(n >= OP_TWO_ARG); }
 
 string OpTree::str(const vector<string> *vars)
 {
@@ -59,7 +30,7 @@ string OpTree::str(const vector<string> *vars)
         return vars == NULL || vars->empty() ? "var"+S(v_nr) : (*vars)[v_nr];
     }
     switch (op) {
-        case 0:       return S(val);
+        case OP_NUMBER: return S(val); // OP_NUMBER == 0
         case OP_NEG:  return "-" + c1->str_b(c1->op >= OP_POW, vars);
         case OP_EXP:  return "exp(" + c1->str(vars) + ")";
         case OP_ERFC: return "erfc(" + c1->str(vars) + ")";
@@ -97,98 +68,29 @@ string OpTree::str(const vector<string> *vars)
     }
 }
 
-string OpTree::ascii_tree(int width, int start, const vector<string> *vars)
-{
-    string node = "???";
-    if (op < 0) {
-        int v_nr = -op-1;
-        node = vars->empty() ? "var"+S(v_nr) : (*vars)[v_nr];
-    }
-    else
-        switch (op) {
-            case 0:       node = S(val); break;
-            case OP_NEG:  node = "NEG";  break;
-            case OP_EXP:  node = "EXP";  break;
-            case OP_ERFC: node = "ERFC"; break;
-            case OP_ERF:  node = "ERF";  break;
-            case OP_SINH: node = "SINH"; break;
-            case OP_COSH: node = "COSH"; break;
-            case OP_TANH: node = "TANH"; break;
-            case OP_SIN:  node = "SIN";  break;
-            case OP_COS:  node = "COS";  break;
-            case OP_TAN:  node = "TAN";  break;
-            case OP_ASIN: node = "ASIN"; break;
-            case OP_ACOS: node = "ACOS"; break;
-            case OP_ATAN: node = "ATAN"; break;
-            case OP_LOG10:node = "LOG";  break;
-            case OP_LN:   node = "LN";   break;
-            case OP_SQRT: node = "SQRT"; break;
-            case OP_POW:  node = "POW";  break;
-            case OP_ADD:  node = "ADD";  break;
-            case OP_SUB:  node = "SUB";  break;
-            case OP_MUL:  node = "MUL";  break;
-            case OP_DIV:  node = "DIV";  break;
-        }
-    int n = (int(node.size()) < width ? start + (width-node.size())/2
-                                 : start);
-    node = string(n, ' ') + node + "\n";
-    if (c1)
-        node += c1->ascii_tree(width/2, start, vars);
-    if (c2)
-        node += c2->ascii_tree(width/2, start+width/2, vars);
-    return node;
-}
 
-OpTree* OpTree::copy() const
+OpTree* OpTree::clone() const
 {
     OpTree *t = new OpTree(*this);
-    if (c1) t->c1 = c1->copy();
-    if (c2) t->c2 = c2->copy();
+    if (c1) t->c1 = c1->clone();
+    if (c2) t->c2 = c2->clone();
     return t;
 }
 
 ////////////////////////////////////////////////////////////////////////////
 
-namespace {
-void do_find_tokens(int tokenID, const_tm_iter_t const &i, vector<string> &vars)
-{
-    for (const_tm_iter_t j = i->children.begin(); j != i->children.end(); ++j) {
-        if (j->value.id() == tokenID) {
-            string v(j->value.begin(), j->value.end());
-            if (find(vars.begin(), vars.end(), v) == vars.end())
-                vars.push_back(v);
-        }
-        else
-            do_find_tokens(tokenID, j, vars);
-    }
-}
-} // anonymous namespace
-
-vector<string> find_tokens_in_ptree(int tokenID, const tree_parse_info<> &info)
-{
-    vector<string> vars;
-    const_tm_iter_t const &root = info.trees.begin();
-    if (root->value.id() == tokenID)
-        vars.push_back(string(root->value.begin(), root->value.end()));
-    else
-        do_find_tokens(tokenID, root, vars);
-    return vars;
-}
-
-
-////////////////////////////////////////////////////////////////////////////
 OpTree* simplify_terms(OpTree *a);
 OpTree* do_multiply(OpTree *a, OpTree *b);
 
 OpTree* do_neg(OpTree *a)
 {
     if (a->op == 0) {
-        double val = - a->val;
+        realt val = - a->val;
         delete a;
         return new OpTree(val);
     }
     else if (a->op == OP_NEG) {
-        OpTree *t = a->c1->copy();
+        OpTree *t = a->c1->clone();
         delete a;
         return t;
     }
@@ -199,7 +101,7 @@ OpTree* do_neg(OpTree *a)
 OpTree* do_add(int op, OpTree *a, OpTree *b)
 {
     if (a->op == 0 && b->op == 0) { // p + q
-        double val = (op == OP_ADD ? a->val + b->val : a->val - b->val);
+        realt val = (op == OP_ADD ? a->val + b->val : a->val - b->val);
         delete a;
         delete b;
         return new OpTree(val);
@@ -252,7 +154,7 @@ OpTree* do_multiply(OpTree *a, OpTree *b)
 {
     if (a->op == 0 && b->op == 0)  // const * const
     {
-        double val = a->val * b->val;
+        realt val = a->val * b->val;
         delete a;
         delete b;
         return new OpTree(val);
@@ -296,7 +198,7 @@ OpTree* do_divide(OpTree *a, OpTree *b)
     //no check for division by zero
     if (a->op == 0 && b->op == 0)
     {
-        double val = a->val / b->val;
+        realt val = a->val / b->val;
         delete a;
         delete b;
         return new OpTree(val);
@@ -317,8 +219,8 @@ OpTree* do_divide(OpTree *a, OpTree *b)
 
 OpTree *do_sqr(OpTree *a)
 {
-    return do_multiply(a, a->copy());
-    //return new OpTree(OP_MUL, a, a->copy());
+    return do_multiply(a, a->clone());
+    //return new OpTree(OP_MUL, a, a->clone());
 }
 
 OpTree *do_oneover(OpTree *a)
@@ -331,7 +233,7 @@ template<typename T>
 OpTree* one_arg_func(OpTree *a, T func, int op)
 {
     if (a->op == 0) {
-        double val = func(a->val);
+        realt val = func(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -343,7 +245,7 @@ OpTree* one_arg_func(OpTree *a, T func, int op)
 OpTree* do_exp(OpTree *a)
 {
     if (a->op == 0) {
-        double val = exp(a->val);
+        realt val = exp(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -354,7 +256,7 @@ OpTree* do_exp(OpTree *a)
 OpTree* do_erf(OpTree *a)
 {
     if (a->op == 0) {
-        double val = erf(a->val);
+        realt val = erf(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -365,7 +267,7 @@ OpTree* do_erf(OpTree *a)
 OpTree* do_erfc(OpTree *a)
 {
     if (a->op == 0) {
-        double val = erfc(a->val);
+        realt val = erfc(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -376,7 +278,7 @@ OpTree* do_erfc(OpTree *a)
 OpTree* do_sqrt(OpTree *a)
 {
     if (a->op == 0) {
-        double val = sqrt(a->val);
+        realt val = sqrt(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -387,7 +289,7 @@ OpTree* do_sqrt(OpTree *a)
 OpTree* do_log10(OpTree *a)
 {
     if (a->op == 0) {
-        double val = log10(a->val);
+        realt val = log10(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -398,7 +300,7 @@ OpTree* do_log10(OpTree *a)
 OpTree* do_ln(OpTree *a)
 {
     if (a->op == 0) {
-        double val = log(a->val);
+        realt val = log(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -409,7 +311,7 @@ OpTree* do_ln(OpTree *a)
 OpTree* do_sinh(OpTree *a)
 {
     if (a->op == 0) {
-        double val = sinh(a->val);
+        realt val = sinh(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -420,7 +322,7 @@ OpTree* do_sinh(OpTree *a)
 OpTree* do_cosh(OpTree *a)
 {
     if (a->op == 0) {
-        double val = cosh(a->val);
+        realt val = cosh(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -431,7 +333,7 @@ OpTree* do_cosh(OpTree *a)
 OpTree* do_tanh(OpTree *a)
 {
     if (a->op == 0) {
-        double val = tanh(a->val);
+        realt val = tanh(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -442,7 +344,7 @@ OpTree* do_tanh(OpTree *a)
 OpTree* do_sin(OpTree *a)
 {
     if (a->op == 0) {
-        double val = sin(a->val);
+        realt val = sin(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -453,7 +355,7 @@ OpTree* do_sin(OpTree *a)
 OpTree* do_cos(OpTree *a)
 {
     if (a->op == 0) {
-        double val = cos(a->val);
+        realt val = cos(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -464,7 +366,7 @@ OpTree* do_cos(OpTree *a)
 OpTree* do_tan(OpTree *a)
 {
     if (a->op == 0) {
-        double val = tan(a->val);
+        realt val = tan(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -475,7 +377,7 @@ OpTree* do_tan(OpTree *a)
 OpTree* do_atan(OpTree *a)
 {
     if (a->op == 0) {
-        double val = atan(a->val);
+        realt val = atan(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -486,7 +388,7 @@ OpTree* do_atan(OpTree *a)
 OpTree* do_asin(OpTree *a)
 {
     if (a->op == 0) {
-        double val = asin(a->val);
+        realt val = asin(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -497,7 +399,7 @@ OpTree* do_asin(OpTree *a)
 OpTree* do_acos(OpTree *a)
 {
     if (a->op == 0) {
-        double val = acos(a->val);
+        realt val = acos(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -508,7 +410,7 @@ OpTree* do_acos(OpTree *a)
 OpTree* do_lgamma(OpTree *a)
 {
     if (a->op == 0) {
-        double val = boost::math::lgamma(a->val);
+        realt val = boost::math::lgamma(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -519,7 +421,7 @@ OpTree* do_lgamma(OpTree *a)
 OpTree* do_digamma(OpTree *a)
 {
     if (a->op == 0) {
-        double val = boost::math::digamma(a->val);
+        realt val = boost::math::digamma(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -530,7 +432,7 @@ OpTree* do_digamma(OpTree *a)
 OpTree* do_abs(OpTree *a)
 {
     if (a->op == 0) {
-        double val = fabs(a->val);
+        realt val = fabs(a->val);
         delete a;
         return new OpTree(val);
     }
@@ -541,7 +443,7 @@ OpTree* do_abs(OpTree *a)
 OpTree* do_pow(OpTree *a, OpTree *b)
 {
     if (a->op == 0 && b->op == 0) {
-        double val = pow(a->val, b->val);
+        realt val = pow(a->val, b->val);
         delete a;
         delete b;
         return new OpTree(val);
@@ -573,7 +475,7 @@ OpTree* do_pow(OpTree *a, OpTree *b)
 OpTree* do_voigt(OpTree *a, OpTree *b)
 {
     if (a->op == 0 && b->op == 0) {
-        double val = humlik(a->val, b->val) / sqrt(M_PI);
+        realt val = humlik(a->val, b->val) / sqrt(M_PI);
         delete a;
         return new OpTree(val);
     }
@@ -584,7 +486,7 @@ OpTree* do_voigt(OpTree *a, OpTree *b)
 OpTree* do_dvoigt_dx(OpTree *a, OpTree *b)
 {
     if (a->op == 0 && b->op == 0) {
-        double val = humdev_dkdx(a->val, b->val) / sqrt(M_PI);
+        realt val = humdev_dkdx(a->val, b->val) / sqrt(M_PI);
         delete a;
         return new OpTree(val);
     }
@@ -595,7 +497,7 @@ OpTree* do_dvoigt_dx(OpTree *a, OpTree *b)
 OpTree* do_dvoigt_dy(OpTree *a, OpTree *b)
 {
     if (a->op == 0 && b->op == 0) {
-        double val = humdev_dkdy(a->val, b->val) / sqrt(M_PI);
+        realt val = humdev_dkdy(a->val, b->val) / sqrt(M_PI);
         delete a;
         return new OpTree(val);
     }
@@ -618,7 +520,7 @@ struct MultFactor
 /// and builds list of nodes with factors, such that tree a is equal to
 /// (v[0]->t)^(v[0]->e) * (v[1]->t)^(v[1]->e) * ...
 void get_factors(OpTree *a, OpTree *expo,
-                 double &constant, vector<MultFactor>& v)
+                 realt &constant, vector<MultFactor>& v)
 {
     if (a->op == OP_ADD || a->op == OP_SUB)
         a = simplify_terms(a);
@@ -630,7 +532,7 @@ void get_factors(OpTree *a, OpTree *expo,
     }
     else if (a->op == OP_DIV) {
         get_factors(a->c1, expo, constant, v);
-        OpTree *expo2 = do_neg(expo->copy());
+        OpTree *expo2 = do_neg(expo->clone());
         get_factors(a->c2, expo2, constant, v);
         delete expo2;
     }
@@ -639,12 +541,12 @@ void get_factors(OpTree *a, OpTree *expo,
         get_factors(new OpTree(-1.), expo, constant, v);
     }
     else if (a->op == OP_SQRT) {
-        OpTree *expo2 = do_multiply(new OpTree(0.5), expo->copy());
+        OpTree *expo2 = do_multiply(new OpTree(0.5), expo->clone());
         get_factors(a->c1, expo2, constant, v);
         delete expo2;
     }
     else if (a->op == OP_POW) {
-        OpTree *expo2 = do_multiply(a->remove_c2(), expo->copy());
+        OpTree *expo2 = do_multiply(a->remove_c2(), expo->clone());
         get_factors(a->c1, expo2, constant, v);
         delete expo2;
     }
@@ -652,12 +554,12 @@ void get_factors(OpTree *a, OpTree *expo,
         bool found = false;
         for (vector<MultFactor>::iterator i = v.begin(); i != v.end(); ++i)
             if (*i->t == *a) {
-                i->e = do_add(i->e, expo->copy());
+                i->e = do_add(i->e, expo->clone());
                 found = true;
                 break;
             }
             if (!found) {
-                v.push_back(MultFactor(a, expo->copy()));
+                v.push_back(MultFactor(a, expo->clone()));
                 return; //don't delete a
             }
     }
@@ -674,7 +576,7 @@ OpTree* simplify_factors(OpTree *a)
 #endif
     vector<MultFactor> v;
     OpTree expo(1.);
-    double constant = 1;
+    realt constant = 1;
     get_factors(a, &expo, constant, v); //deletes a
 #ifdef DEBUG_SIMPLIFY
     cout << "simplify_factors(): [.] {" << constant << "} ";
@@ -753,12 +655,12 @@ OpTree* simplify_factors(OpTree *a)
 struct MultTerm
 {
     OpTree *t;
-    double k;
-    MultTerm(OpTree *t_, double k_) : t(t_), k(k_) {}
+    realt k;
+    MultTerm(OpTree *t_, realt k_) : t(t_), k(k_) {}
     void clear() { delete t; t=0; }
 };
 
-void get_terms(OpTree *a, double multiplier, vector<MultTerm> &v)
+void get_terms(OpTree *a, realt multiplier, vector<MultTerm> &v)
 {
     if (a->op == OP_MUL || a->op == OP_DIV || a->op == OP_SQRT
             || a->op == OP_POW)
@@ -839,14 +741,14 @@ OpTree* simplify_terms(OpTree *a)
 #endif
 
     // sin^2(x) + cos^2(x) = 1
-    double to_add = 0.;
+    realt to_add = 0.;
     for (vector<MultTerm>::iterator i = v.begin(); i != v.end(); ++i)
         if (i->t && i->t->op == OP_POW && i->t->c1->op == OP_SIN
                 && i->t->c2->op == 0 && is_eq(i->t->c2->val, 2.))
             for (vector<MultTerm>::iterator j = v.begin(); j != v.end(); ++j)
                 if (j->t && j->t->op == OP_POW && j->t->c1->op == OP_COS
                         && j->t->c2->op == 0 && is_eq(j->t->c2->val, 2.)) {
-                    double k = j->k;
+                    realt k = j->k;
                     i->k -= k;
                     j->clear();
                     to_add += k;
@@ -875,187 +777,225 @@ OpTree* simplify_terms(OpTree *a)
 
 ////////////////////////////////////////////////////////////////////////////
 
-fp get_constant_value(string const &s)
+
+/// simplify exportable formula, i.e. mathematical function f(x),
+/// without variables other than x
+string simplify_formula(string const &formula)
 {
-    if (s == "pi")
-        return M_PI;
-    else if (s[0] == '{') {
-        assert(*(s.end()-1) == '}');
-        string expr(s.begin()+1, s.end()-1);
-        Data const* data = 0;
-        string::size_type in_pos = expr.find(" in ");
-        if (in_pos != string::npos && in_pos+4 < expr.size()) {
-            string in_expr(expr, in_pos+4);
-            int n;
-            if (parse(in_expr.c_str(),
-                      !space_p >> '@' >> uint_p[assign_a(n)] >> !space_p
-                     ).full) {
-                data = AL->get_data(n);
-                expr.resize(in_pos);
-            }
-            else
-                throw ExecuteError("Syntax error near: `" + in_expr + "'");
-        }
-        else if (AL->get_dm_count() == 1)
-            data = AL->get_data(0);
-        return get_transform_expression_value(expr, data);
+    Lexer lex(formula.c_str());
+    ExpressionParser ep(NULL);
+    // if the formula has not-expanded-functions, like Voigt(2,3,4,5),
+    // it throws SyntaxError
+    try {
+        ep.parse_expr(lex, -1, NULL, NULL, true);
     }
-    else {
-        fp val = strtod(s.c_str(), 0);
-        //if (val != 0. && fabs(val) < epsilon)
-        //    AL->warn("Warning: Numeric literal 0 < |" + s + "| < epsilon="
-        //            + S(epsilon) + ".");
-        return val;
+    catch (fityk::SyntaxError&) {
+        return formula;
     }
+    // derivatives are calculated only as a side effect
+    vector<OpTree*> trees = prepare_ast_with_der(ep.vm(), 1);
+    vector<string> vars(1, "x");
+    string simplified = trees.back()->str(&vars);
+    purge_all_elements(trees);
+    return simplified;
+}
+
+/// debug utility, shows symbolic derivatives of given formula
+void get_derivatives_str(const char* formula, string& result)
+{
+    Lexer lex(formula);
+    ExpressionParser ep(NULL);
+    vector<string> vars;
+    ep.parse_expr(lex, -1, NULL, &vars);
+    vector<OpTree*> trees = prepare_ast_with_der(ep.vm(), vars.size());
+
+    result += "f(" + join_vector(vars,", ") + ") = " + trees.back()->str(&vars);
+    for (size_t i = 0; i != vars.size(); ++i)
+        result += "\ndf / d " + vars[i] + " = " + trees[i]->str(&vars);
+    purge_all_elements(trees);
 }
 
 /// returns array of trees,
 /// first n=vars.size() derivatives and the last tree for value
-vector<OpTree*> calculate_deriv(const_tm_iter_t const &i,
-                                vector<string> const &vars)
+static
+vector<OpTree*> calculate_deriv(vector<int>::const_iterator &i, int len,
+                                const VMData& vm)
 {
-    int len = vars.size();
-    vector<OpTree*> results(len + 1);
-    string s(i->value.begin(), i->value.end());
+  vector<OpTree*> results(len + 1, (OpTree*) NULL);
+  --i;
+  assert(i >= vm.code().begin());
 
-    if (i->value.id() == FuncGrammar::real_constID)
-    {
-        assert(s.size() > 0);
+  if (*i < 0) {
+      --i;
+      assert(i >= vm.code().begin());
+      assert(*i == OP_NUMBER || *i == OP_SYMBOL);
+  }
+
+  switch (*i) {
+
+    case OP_NUMBER: {
         for (int k = 0; k < len; ++k)
             results[k] = new OpTree(0.);
-        double v = get_constant_value(s);
-        results[len] = new OpTree(v);
+        int negative_idx = *(i+1);
+        int idx = -1 - negative_idx;
+        assert(is_index(idx, vm.numbers()));
+        realt value = vm.numbers()[idx];
+        results[len] = new OpTree(value);
+        break;
     }
 
-    else if (i->value.id() == FuncGrammar::variableID)
-    {
-        for (int k = 0; k < len; ++k)
-            if (s == vars[k]) {
-                results[k] = new OpTree(1.);
-                results[len] = new OpTree(k, s);
-            }
-            else
-                results[k] = new OpTree(0.);
-    }
-
-    else if (i->value.id() == FuncGrammar::exptokenID)
-    {
-        if (i->children.size() == 1) {
-            vector<OpTree*> arg = calculate_deriv(i->children.begin(), vars);
-            OpTree* (* do_op)(OpTree *) = 0;
-            OpTree* der = 0;
-            OpTree* larg = arg.back()->copy();
-            if (s == "sqrt") {
-                der = do_divide(new OpTree(0.5), do_sqrt(larg));
-                do_op = do_sqrt;
-            }
-            else if (s == "exp") {
-                der = do_exp(larg);
-                do_op = do_exp;
-            }
-            else if (s == "erfc") {
-                // d/dz erfc(z) = -2/sqrt(pi) * exp(-z^2)
-                der = do_multiply(do_exp(do_neg(do_sqr(larg))),
-                                  new OpTree(-2/sqrt(M_PI)));
-                do_op = do_erfc;
-            }
-            else if (s == "erf") {
-                // d/dz erf(z) =  2/sqrt(pi) * exp(-z^2)
-                der = do_multiply(do_exp(do_neg(do_sqr(larg))),
-                                  new OpTree(2/sqrt(M_PI)));
-                do_op = do_erf;
-            }
-            else if (s == "log10") {
-                OpTree *ln_10 = do_ln(new OpTree(10.));
-                der = do_oneover(do_multiply(larg, ln_10));
-                do_op = do_log10;
-            }
-            else if (s == "ln") {
-                der = do_oneover(larg);
-                do_op = do_ln;
-            }
-            else if (s == "sinh") {
-                der = do_cosh(larg);
-                do_op = do_sinh;
-            }
-            else if (s == "cosh") {
-                der = do_sinh(larg);
-                do_op = do_cosh;
-            }
-            else if (s == "tanh") {
-                der = do_oneover(do_sqr(do_cosh(larg)));
-                do_op = do_tanh;
-            }
-            else if (s == "sin") {
-                der = do_cos(larg);
-                do_op = do_sin;
-            }
-            else if (s == "cos") {
-                der = do_neg(do_sin(larg));
-                do_op = do_cos;
-            }
-            else if (s == "tan") {
-                der = do_oneover(do_sqr(do_cos(larg)));
-                do_op = do_tan;
-            }
-            else if (s == "atan") {
-                der = do_oneover(do_add(new OpTree(1.), do_sqr(larg)));
-                do_op = do_atan;
-            }
-            else if (s == "asin") {
-                OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
-                der = do_oneover(do_sqrt(root_arg));
-                do_op = do_asin;
-            }
-            else if (s == "acos") {
-                OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
-                der = do_divide(new OpTree(-1.), do_sqrt(root_arg));
-                do_op = do_acos;
-            }
-            else if (s == "lgamma") {
-                der = do_digamma(larg);
-                do_op = do_lgamma;
-            }
-            else if (s == "abs") {
-                der = do_divide(do_abs(larg), larg->copy());
-                do_op = do_abs;
-            }
-            else
-                assert(0);
-            for (int k = 0; k < len; ++k)
-                results[k] = do_multiply(der->copy(), arg[k]);
-            delete der;
-            results[len] = (*do_op)(arg[len]);
+    case OP_SYMBOL: {
+        int negative_idx = *(i+1);
+        int idx = -1 - negative_idx;
+        assert(idx <= len);
+        for (int k = 0; k < len; ++k) {
+            realt der_value = (k == idx ? 1. : 0.);
+            results[k] = new OpTree(der_value);
         }
-        else if (i->children.size() == 2) {
-            vector<OpTree*>
-                left = calculate_deriv(i->children.begin(), vars),
-                right = calculate_deriv(i->children.begin() + 1, vars);
-            OpTree *d1=0, *d2=0;
-            if (s == "voigt") {
-                d1 = do_dvoigt_dx(left[len]->copy(), right[len]->copy());
-                d2 = do_dvoigt_dy(left[len]->copy(), right[len]->copy());
-                for (int k = 0; k < len; ++k) {
-                    results[k] = do_add(do_multiply(d1->copy(), left[k]),
-                                        do_multiply(d2->copy(), right[k]));
-                }
-                results[len] = do_voigt(left[len], right[len]);
-            }
-            else
-                assert(0);
-            delete d1;
-            delete d2;
+        results[len] = new OpTree(NULL, idx);
+        break;
+    }
+
+    case OP_X: {
+        int n = len - 1;
+        // the rest is the same as in OP_SYMBOL
+        for (int k = 0; k < len; ++k) {
+            realt der_value = (k == n ? 1. : 0.);
+            results[k] = new OpTree(der_value);
+        }
+        results[len] = new OpTree(NULL, n);
+        break;
+    }
+
+
+    case OP_SQRT:
+    case OP_EXP:
+    case OP_ERFC:
+    case OP_ERF:
+    case OP_LOG10:
+    case OP_LN:
+    case OP_SINH:
+    case OP_COSH:
+    case OP_TANH:
+    case OP_SIN:
+    case OP_COS:
+    case OP_TAN:
+    case OP_ASIN:
+    case OP_ACOS:
+    case OP_ATAN:
+    case OP_LGAMMA:
+    case OP_ABS:
+    {
+        int op = *i;
+        vector<OpTree*> arg = calculate_deriv(i, len, vm);
+        OpTree* (* do_op)(OpTree *) = NULL;
+        OpTree* der = 0;
+        OpTree* larg = arg.back()->clone();
+        if (op == OP_SQRT) {
+            der = do_divide(new OpTree(0.5), do_sqrt(larg));
+            do_op = do_sqrt;
+        }
+        else if (op == OP_EXP) {
+            der = do_exp(larg);
+            do_op = do_exp;
+        }
+        else if (op == OP_ERFC) {
+            // d/dz erfc(z) = -2/sqrt(pi) * exp(-z^2)
+            der = do_multiply(do_exp(do_neg(do_sqr(larg))),
+                              new OpTree(-2/sqrt(M_PI)));
+            do_op = do_erfc;
+        }
+        else if (op == OP_ERF) {
+            // d/dz erf(z) =  2/sqrt(pi) * exp(-z^2)
+            der = do_multiply(do_exp(do_neg(do_sqr(larg))),
+                              new OpTree(2/sqrt(M_PI)));
+            do_op = do_erf;
+        }
+        else if (op == OP_LOG10) {
+            OpTree *ln_10 = do_ln(new OpTree(10.));
+            der = do_oneover(do_multiply(larg, ln_10));
+            do_op = do_log10;
+        }
+        else if (op == OP_LN) {
+            der = do_oneover(larg);
+            do_op = do_ln;
+        }
+        else if (op == OP_SINH) {
+            der = do_cosh(larg);
+            do_op = do_sinh;
+        }
+        else if (op == OP_COSH) {
+            der = do_sinh(larg);
+            do_op = do_cosh;
+        }
+        else if (op == OP_TANH) {
+            der = do_oneover(do_sqr(do_cosh(larg)));
+            do_op = do_tanh;
+        }
+        else if (op == OP_SIN) {
+            der = do_cos(larg);
+            do_op = do_sin;
+        }
+        else if (op == OP_COS) {
+            der = do_neg(do_sin(larg));
+            do_op = do_cos;
+        }
+        else if (op == OP_TAN) {
+            der = do_oneover(do_sqr(do_cos(larg)));
+            do_op = do_tan;
+        }
+        else if (op == OP_ASIN) {
+            OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
+            der = do_oneover(do_sqrt(root_arg));
+            do_op = do_asin;
+        }
+        else if (op == OP_ACOS) {
+            OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
+            der = do_divide(new OpTree(-1.), do_sqrt(root_arg));
+            do_op = do_acos;
+        }
+        else if (op == OP_ATAN) {
+            der = do_oneover(do_add(new OpTree(1.), do_sqr(larg)));
+            do_op = do_atan;
+        }
+        else if (op == OP_LGAMMA) {
+            der = do_digamma(larg);
+            do_op = do_lgamma;
+        }
+        else if (op == OP_ABS) {
+            der = do_divide(do_abs(larg), larg->clone());
+            do_op = do_abs;
         }
         else
             assert(0);
+        for (int k = 0; k < len; ++k)
+            results[k] = do_multiply(der->clone(), arg[k]);
+        delete der;
+        results[len] = (*do_op)(arg[len]);
+        break;
     }
 
-    else if (i->value.id() == FuncGrammar::signargID)
+    case OP_VOIGT:
     {
-        assert(s == "^");
-        assert(i->children.size() == 2);
-        vector<OpTree*> left = calculate_deriv(i->children.begin(), vars);
-        vector<OpTree*> right = calculate_deriv(i->children.begin() + 1, vars);
+        int op = *i;
+        vector<OpTree*> right = calculate_deriv(i, len, vm);
+        vector<OpTree*> left = calculate_deriv(i, len, vm);
+        assert (op == OP_VOIGT);
+        OpTree *d1 = do_dvoigt_dx(left[len]->clone(), right[len]->clone());
+        OpTree *d2 = do_dvoigt_dy(left[len]->clone(), right[len]->clone());
+        for (int k = 0; k < len; ++k) {
+            results[k] = do_add(do_multiply(d1->clone(), left[k]),
+                                do_multiply(d2->clone(), right[k]));
+        }
+        results[len] = do_voigt(left[len], right[len]);
+        delete d1;
+        delete d2;
+        break;
+    }
+
+    case OP_POW: {
+        vector<OpTree*> right = calculate_deriv(i, len, vm);
+        vector<OpTree*> left = calculate_deriv(i, len, vm);
         // this special case is needed, because in cases like -2^4
         // there was a problem with logarithm in formula below
         if (left[len]->op == 0 && right[len]->op == 0) {
@@ -1078,85 +1018,125 @@ vector<OpTree*> calculate_deriv(const_tm_iter_t const &i,
                 //        b(x)            b(x)  b(x) a'(x)
                 //    (a(x)   )'    = a(x)     (---------- + ln(a(x)) b'(x))
                 //                                 a(x)
-                OpTree *pow_a_b = do_pow(a->copy(), b->copy());
-                OpTree *term1 = do_divide(do_multiply(b->copy(),ap), a->copy());
-                OpTree *term2 = do_multiply(do_ln(a->copy()), bp);
+                OpTree *pow_a_b = do_pow(a->clone(), b->clone());
+                OpTree *term1 = do_divide(do_multiply(b->clone(), ap),
+                                          a->clone());
+                OpTree *term2 = do_multiply(do_ln(a->clone()), bp);
                 results[k] = do_multiply(pow_a_b, do_add(term1, term2));
             }
             results[len] = do_pow(left[len], right[len]);
         }
+        break;
     }
 
-    else if (i->value.id() == FuncGrammar::factorID)
-    {
-        assert (s == "-");
-        vector<OpTree*> arg = calculate_deriv(i->children.begin(), vars);
+    case OP_NEG: {
+        vector<OpTree*> arg = calculate_deriv(i, len, vm);
         for (int k = 0; k < len+1; ++k)
             results[k] = do_neg(arg[k]);
+        break;
     }
 
-    else if (i->value.id() == FuncGrammar::termID)
-    {
-        assert(s == "*" || s == "/");
-        assert(i->children.size() == 2);
-        int op = (s == "*" ? OP_MUL : OP_DIV);
-        vector<OpTree*> left = calculate_deriv(i->children.begin(), vars);
-        vector<OpTree*> right = calculate_deriv(i->children.begin() + 1, vars);
+    case OP_MUL: {
+        vector<OpTree*> right = calculate_deriv(i, len, vm);
+        vector<OpTree*> left = calculate_deriv(i, len, vm);
         for (int k = 0; k < len; ++k) {
+            // a*b' + a'*b
             OpTree *a = left[len],
                    *b = right[len],
                    *ap = left[k],
                    *bp = right[k];
-            if (op == OP_MUL) { // a*b' + a'*b
-                results[k] = do_add(do_multiply(a->copy(), bp),
-                                    do_multiply(ap,  b->copy()));
-            }
-            else { //OP_DIV    (a'*b - b'*a) / (b*b)
-                OpTree *upper = do_sub(do_multiply(ap, b->copy()),
-                                       do_multiply(bp, a->copy()));
-                results[k] = do_divide(upper, do_sqr(b->copy()));
-            }
+            results[k] = do_add(do_multiply(a->clone(), bp),
+                                do_multiply(ap, b->clone()));
         }
-        results[len] = (op == OP_MUL ? do_multiply(left[len], right[len])
-                                     : do_divide(left[len], right[len]));
+        results[len] = do_multiply(left[len], right[len]);
+        break;
     }
 
-    else if (i->value.id() == FuncGrammar::expressionID)
-    {
-        assert(s == "+" || s == "-");
-        assert(i->children.size() == 2);
-        vector<OpTree*> left = calculate_deriv(i->children.begin(), vars);
-        vector<OpTree*> right = calculate_deriv(i->children.begin() + 1, vars);
+    case OP_DIV: {
+        vector<OpTree*> right = calculate_deriv(i, len, vm);
+        vector<OpTree*> left = calculate_deriv(i, len, vm);
+        for (int k = 0; k < len; ++k) {
+            // (a'*b - b'*a) / (b*b)
+            OpTree *a = left[len],
+                   *b = right[len],
+                   *ap = left[k],
+                   *bp = right[k];
+            OpTree *upper = do_sub(do_multiply(ap, b->clone()),
+                                   do_multiply(bp, a->clone()));
+            results[k] = do_divide(upper, do_sqr(b->clone()));
+        }
+        results[len] = do_divide(left[len], right[len]);
+        break;
+    }
+
+    case OP_ADD: {
+        vector<OpTree*> right = calculate_deriv(i, len, vm);
+        vector<OpTree*> left = calculate_deriv(i, len, vm);
         for (int k = 0; k < len+1; ++k)
-            results[k] = (s == "+" ? do_add(left[k], right[k])
-                                   : do_sub(left[k], right[k]));
+            results[k] = do_add(left[k], right[k]);
+        break;
     }
 
-    else
-        assert(0); // error
+    case OP_SUB: {
+        vector<OpTree*> right = calculate_deriv(i, len, vm);
+        vector<OpTree*> left = calculate_deriv(i, len, vm);
+        for (int k = 0; k < len+1; ++k)
+            results[k] = do_sub(left[k], right[k]);
+        break;
+    }
 
-    for (int k = 0; k < len+1; ++k)
-        results[k] = simplify_terms(results[k]);
+    default:
+        purge_all_elements(results);
+        throw ExecuteError("`" + op2str(*i) + "' is not allowed for "
+                           "variables/functions");
+    }
+  assert(results[0] != NULL);
+  assert(results[len] != NULL);
 
-    return results;
+  for (int k = 0; k < len+1; ++k)
+      results[k] = simplify_terms(results[k]);
+
+  //printf("%s\n", results[len]->str().c_str());
+  return results;
 }
 
-
-/// debug utility, shows symbolic derivatives of given formula
-size_t get_derivatives_str(const char* formula, string& result)
+vector<OpTree*> prepare_ast_with_der(const VMData& vm, int len)
 {
-    tree_parse_info<> info = ast_parse(formula, FuncG, space_p);
-    printf("%d, %s\n", (int) info.length, formula);
-    if (!info.match)
-        throw ExecuteError("Can't parse formula: " + string(formula));
-    const_tm_iter_t const &root = info.trees.begin();
-    vector<string> vars = find_tokens_in_ptree(FuncGrammar::variableID, info);
-    vector<OpTree*> results = calculate_deriv(root, vars);
-    result += "f(" + join_vector(vars, ", ") + ") = "
-              + results.back()->str(&vars);
-    for (size_t i = 0; i != vars.size(); ++i)
-        result += "\ndf / d " + vars[i] + " = " + results[i]->str(&vars);
-    purge_all_elements(results);
-    return info.length;
+    assert(!vm.code().empty());
+    const_cast<VMData&>(vm).flip_indices();
+    vector<int>::const_iterator iter = vm.code().end();
+    vector<OpTree*> r = calculate_deriv(iter, len, vm);
+    assert (iter == vm.code().begin());
+    const_cast<VMData&>(vm).flip_indices();
+    return r;
+}
+
+void add_bytecode_from_tree(const OpTree* tree, const vector<int> &symbol_map,
+                            VMData& vm)
+{
+    int op = tree->op;
+    if (op < 0) {
+        size_t n = -op-1;
+        if (n == symbol_map.size()) {
+            vm.append_code(OP_X);
+        }
+        else {
+            assert(is_index(n, symbol_map));
+            vm.append_code(OP_SYMBOL);
+            vm.append_code(symbol_map[n]);
+        }
+    }
+    else if (op == 0) {
+        vm.append_number(tree->val);
+    }
+    else if (op >= OP_ONE_ARG && op < OP_TWO_ARG) { //one argument
+        add_bytecode_from_tree(tree->c1, symbol_map, vm);
+        vm.append_code(op);
+    }
+    else if (op >= OP_TWO_ARG) { //two arguments
+        add_bytecode_from_tree(tree->c1, symbol_map, vm);
+        add_bytecode_from_tree(tree->c2, symbol_map, vm);
+        vm.append_code(op);
+    }
 }
 
