@@ -1,7 +1,5 @@
 // This file is part of fityk program. Copyright (C) 2009 Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: about.h 503 2009-06-09 15:47:31Z wojdyr $
-//
 // AboutDlg: "about..." dialog
 
 #ifndef FITYK_WX_ABOUT_H_
@@ -16,4 +14,11 @@ public:
     AboutDlg(wxWindow* parent);
 };
 
+class AboutReleaseDlg : public wxDialog
+{
+public:
+    AboutReleaseDlg(wxWindow* parent);
+    void OnCheckbox(wxCommandEvent& event);
+};
+
 #endif // FITYK_WX_ABOUT_H_
