@@ -1,6 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: app.h 616 2009-12-29 16:29:41Z wojdyr $
 
 #ifndef FITYK__WX_APP__H__
 #define FITYK__WX_APP__H__
@@ -13,10 +12,6 @@ class wxCmdLineParser;
 class FApp: public wxApp
 {
 public:
-    // default config name
-    std::string conf_filename;
-    // alternative config name
-    std::string alt_conf_filename;
     // directory for (named by user) config files
     wxString config_dir;
 
@@ -28,7 +23,8 @@ private:
     void process_argv(wxCmdLineParser &cmdLineParser);
 };
 
-wxString get_help_url(wxString const& name);
+wxString get_help_url(const wxString& name);
+wxString get_sample_path(const wxString& name);
 
 DECLARE_APP(FApp)
 
