@@ -1,10 +1,10 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: dataedit.cpp 632 2010-01-04 00:01:00Z wojdyr $
 
 #include <wx/wx.h>
 #include <wx/statline.h>
 #include <wx/hyperlink.h>
+#include <fstream>
 
 #include "dataedit.h"
 
@@ -13,7 +13,6 @@
 #include "dialogs.h" //export_data_dlg()
 #include "../data.h" // Data, Point
 #include "../logic.h"
-#include "../datatrans.h" //compile_data_transformation()
 
 using namespace std;
 
@@ -21,80 +20,95 @@ using namespace std;
 // ';' will be replaced by line break
 static const char *default_transforms[] = {
 
-"std.dev.=1||"
+"std.dev.=1"
+"||"
 "|s=1"
 "|Y",
 
-"std.dev.=sqrt(y)||std.dev. = sqrt(y) (or 1 if y<1)"
-"|sqrt(max2(1,y))"
+"std.dev.=sqrt(y)"
+"||std.dev. = sqrt(y) (or 1 if y<1)"
+"|s=sqrt(max2(1,y))"
 "|Y",
 
-"integrate||"
-"|Y[0] = 0"
-";Y[1...] = Y[n-1] + (x[n] - x[n-1]) * (y[n-1] + y[n]) / 2"
+"integrate"
+"||"
+"|Y = Y[n-1] + (x[n] - x[n-1]) * (y[n-1] + y[n]) / 2"
 "|Y",
 
-"differentiate||compute numerical derivative f'(x)"
-"|Y[...-1] = (y[n+1]-y[n])/(x[n+1]-x[n])"
-";X[...-1] = (x[n+1]+x[n])/2"
+"differentiate"
+"||compute numerical derivative f'(x)"
+"|Y = (y[n+1]-y[n])/(x[n+1]-x[n])"
+";X = (x[n+1]+x[n])/2"
 ";M=M-1"
 ";S = sqrt(max2(1,y))"
 "|Y",
 
-"accumulate||Accumulate y of data and adjust std. dev."
-"|Y[1...] = Y[n-1] + y[n]"
+"accumulate"
+"||Accumulate y of data and adjust std. dev."
+"|Y = Y[n-1] + y[n]"
 ";S = sqrt(max2(1,y))"
 "|Y",
 
-"normalize area||"
+"normalize area"
+"||"
 "divide all Y (and std. dev.) values by the current data area"
 " (it gives unit area)"
 "|Y = y/darea(y), S = s / darea(y)"
 "|Y",
 
-"reduce 2x||join every two adjacent points"
-"|X[...-1] = (x[n]+x[n+1])/2"
-";Y[...-1] = y[n]+y[n+1]"
-";S[...-1] = sqrt(s[n]^2+s[n]^2)"
-";delete(n%2==1)"
+"reduce 2x"
+"||join every two adjacent points"
+"|X = (x[n]+x[n+1])/2"
+";Y = y[n]+y[n+1]"
+";S = s[n]+s[n+1]"
+";delete(mod(n,2)==1)"
 "|Y",
 
-"equilibrate step||make equal step, keep the number of points"
-"|X = x[0] + n * (x[M-1]-x[0]) / (M-1), Y = y[x=X], S = s[x=X], A = a[x=X]"
+"equilibrate step"
+"||make equal step, keep the number of points"
+"|X = x[0] + n * (x[M-1]-x[0]) / (M-1), Y = y[index(X)], S = s[index(X)], A = a[index(X)]"
 "|Y",
 
-"zero negative y||zero the Y value; of points with negative Y"
+"zero negative y"
+"||zero the Y value; of points with negative Y"
 "|Y=max2(y,0)"
 "|Y",
 
-"clear inactive||delete inactive points"
+"clear inactive"
+"||delete inactive points"
 "|delete(not a)"
 "|Y",
 
-"swap axes||Swap X and Y axes and adjust std. dev."
+"swap axes"
+"||Swap X and Y axes and adjust std. dev."
 "|Y=x , X=y , S=sqrt(max2(1,Y))"
 "|N",
 
-"generate sinusoid||replaces current data with sinusoid"
+"generate sinusoid"
+"||replaces current data with sinusoid"
 "|M=2000"
 ";x=n/100"
 ";y=sin(x)"
 ";s=1"
 "|N",
 
-"invert||inverts y value of points"
+"invert"
+"||inverts y value of points"
 "|Y=-y"
 "|N",
 
-"activate all||activate all data points"
+"activate all"
+"||activate all data points"
 "|a=true"
 "|N",
 
-"Q -> 2theta(Cu)||rescale X axis (for powder diffraction patterns)"
+"Q -> 2theta(Cu)"
+"||rescale X axis (for powder diffraction patterns)"
 "|X = asin(x/(4*pi)*1.54051) * 2*180/pi"
 "|N",
 
-"2theta(Cu) -> Q||rescale X axis (for powder diffraction patterns)"
+"2theta(Cu) -> Q"
+"||rescale X axis (for powder diffraction patterns)"
 "|X = 4*pi * sin(x/2*pi/180) / 1.54051"
 "|N"
 };
@@ -152,6 +166,7 @@ void EditTransDlg::init()
                     wxSizerFlags().Border(wxLEFT|wxRIGHT|wxTOP));
     wxBoxSizer *lh_sizer = new wxBoxSizer(wxHORIZONTAL);
     trans_list = new wxCheckListBox(left_panel, -1);
+    trans_list->SetMinSize(wxSize(200, -1));
     lh_sizer->Add(trans_list, wxSizerFlags(1).Expand().Border());
     // buttons on the right of the list
     wxBoxSizer *lbutton_sizer = new wxBoxSizer(wxVERTICAL);
@@ -165,16 +180,6 @@ void EditTransDlg::init()
     lbutton_sizer->Add(down_btn, wxSizerFlags().Border());
     lh_sizer->Add(lbutton_sizer, wxSizerFlags().Center());
     left_sizer->Add(lh_sizer, wxSizerFlags(1).Expand());
-    // buttons at the bottom of the list
-    wxBoxSizer *bb_sizer = new wxBoxSizer(wxHORIZONTAL);
-    save_btn = new wxButton(left_panel, wxID_SAVE);
-    bb_sizer->Add(save_btn, wxSizerFlags().Border());
-    revert_btn = new wxButton(left_panel, wxID_REVERT_TO_SAVED);
-    bb_sizer->Add(revert_btn, wxSizerFlags().Border());
-    todefault_btn = new wxButton(left_panel, -1, wxT("Restore &to Default"));
-    todefault_btn->SetToolTip(wxT("Load default transformations."));
-    bb_sizer->Add(todefault_btn, wxSizerFlags().Border());
-    left_sizer->Add(bb_sizer, wxSizerFlags());
     left_panel->SetSizerAndFit(left_sizer);
 
     // right side of the dialog
@@ -197,14 +202,14 @@ void EditTransDlg::init()
     wxBoxSizer *cl_sizer = new wxBoxSizer(wxHORIZONTAL);
     cl_sizer->Add(new wxStaticText(right_panel, -1, wxT("Code:")),
                   wxSizerFlags().Border().Center());
-    wxString help_url = get_help_url(wxT("ref.html"))
+    wxString help_url = get_help_url(wxT("data.html"))
 #ifndef __WXMSW__
           // wxMSW converts URI back to filename, and it can't handle #fragment
-          + wxT("#data-transformations")
+          + wxT("#data-point-transformations")
 #endif
           ;
     wxHyperlinkCtrl *help_ctrl = new wxHyperlinkCtrl(right_panel, -1,
-                                                  wxT("see syntax reference"),
+                                                  wxT("syntax reference"),
                                                   help_url);
     cl_sizer->AddStretchSpacer();
     cl_sizer->Add(help_ctrl, wxSizerFlags().Center().Border(wxRIGHT));
@@ -238,8 +243,20 @@ void EditTransDlg::init()
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
     top_sizer->Add(splitter, wxSizerFlags(1).Expand());
     top_sizer->Add (new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 10);
-    top_sizer->Add(new wxButton(this, wxID_CLOSE, wxT("&Close")),
-                   wxSizerFlags().Border().Center());
+
+    // buttons at the bottom of the dialog
+    wxBoxSizer *bb_sizer = new wxBoxSizer(wxHORIZONTAL);
+    save_btn = new wxButton(this, wxID_SAVE);
+    bb_sizer->Add(save_btn, wxSizerFlags().Border());
+    revert_btn = new wxButton(this, wxID_REVERT_TO_SAVED);
+    bb_sizer->Add(revert_btn, wxSizerFlags().Border());
+    todefault_btn = new wxButton(this, -1, wxT("Restore &to Default"));
+    todefault_btn->SetToolTip(wxT("Load default transformations."));
+    bb_sizer->Add(todefault_btn, wxSizerFlags().Border());
+    bb_sizer->AddStretchSpacer();
+    bb_sizer->Add(new wxButton(this, wxID_CLOSE, wxT("&Close")),
+                  wxSizerFlags().Border().Right());
+    top_sizer->Add(bb_sizer, wxSizerFlags().Expand());
     SetSizerAndFit(top_sizer);
     SetEscapeId(wxID_CLOSE);
 
@@ -474,23 +491,20 @@ void EditTransDlg::OnListItemToggled(wxCommandEvent& event)
 
 void EditTransDlg::execute_tranform(string const& code)
 {
-    string appendix = frame->get_in_datasets();
-    string t = get_code(code, appendix);
-    ftk->exec(t);
+    string t = conv_code_to_one_line(code);
+    ftk->exec(frame->get_datasets() + t);
 }
 
 bool EditTransDlg::update_apply_button()
 {
     string code = wx2s(code_tc->GetValue());
-    string text = get_code(code, "");
-    // to check the syntax, make one command by s/;/,/g
-    replace(text.begin(), text.end(), ';', ',');
-    bool ok = compile_data_transformation(text);
+    string text = conv_code_to_one_line(code);
+    bool ok = ftk->get_ui()->check_syntax(text);
     apply_btn->Enable(ok);
     return ok;
 }
 
-string EditTransDlg::get_code(string const& code, string const& appendix)
+string EditTransDlg::conv_code_to_one_line(string const& code)
 {
     string cmd;
     const char* p = code.c_str();
@@ -510,7 +524,7 @@ string EditTransDlg::get_code(string const& code, string const& appendix)
             ++p;
         if (!cmd.empty())
             cmd += "; ";
-        cmd += string(start, p) + appendix;
+        cmd += string(start, p);
     }
     return cmd;
 }
