@@ -1,10 +1,7 @@
 // Author: Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: powdifpat.cpp 750 2010-06-16 23:27:30Z wojdyr $
 
 //TODO:
-// finish fill_forms()
-// support non-tetraghonal phases
 //
 // deconvolution of instrumental profile
 // buffer the plot with data
@@ -96,7 +93,7 @@ class PlotWithLines : public PlotWithTics
 public:
     PlotWithLines(wxWindow* parent, PhasePanel *phase_panel,
                   PowderBook *powder_book);
-    void draw(wxDC &dc, bool);
+    virtual void draw(wxDC &dc, bool);
 
 private:
     void draw_active_data(wxDC& dc);
@@ -127,6 +124,7 @@ public:
     void OnAtomsChanged(wxCommandEvent& event);
     void set_phase(string const& name, CelFile const& cel);
     const Crystal& get_crystal() const { return cr_; }
+    Crystal& get_crystal() { return cr_; }
     bool editing_atoms() const { return editing_atoms_; }
     int get_selected_hkl() const { return hkl_list->GetSelection(); }
 
@@ -150,9 +148,9 @@ private:
 
     void enable_parameter_fields();
     void update_disabled_parameters();
-    void update_miller_indices();
+    void update_miller_indices(bool sg_changed);
     void set_ortho_angles();
-    void change_space_group();
+    void update_space_group_ui();
 
     DISALLOW_COPY_AND_ASSIGN(PhasePanel);
 };
@@ -175,11 +173,22 @@ LockableRealCtrl *addMaybeRealCtrl(wxWindow *parent, wxString const& label,
 
 } // anonymous namespace
 
+#ifndef wxTBK_BUTTONBAR // for wx < 2.9
+#define wxTBK_BUTTONBAR wxBK_BUTTONBAR
+#endif
+
 vector<PowderBook::PhasePanelExtraData> PowderBook::phase_desc;
 int PowderBook::xaxis_sel = 0;
 
 PowderBook::PowderBook(wxWindow* parent, wxWindowID id)
-    : wxListbook(parent, id), x_min(10), x_max(150), y_max(1000), data(NULL)
+    : wxToolbook(parent, id, wxDefaultPosition, wxDefaultSize,
+#ifdef __WXMAC__
+                 wxTBK_BUTTONBAR
+#else
+                 wxBK_LEFT
+#endif
+                 ),
+      x_min(10), x_max(150), y_max(1000), data(NULL)
 {
 #if !STANDALONE_POWDIFPAT
     int data_nr = frame->get_focused_data_index();
@@ -187,7 +196,10 @@ PowderBook::PowderBook(wxWindow* parent, wxWindowID id)
         data = ftk->get_data(data_nr);
         x_min = data->get_x_min();
         x_max = data->get_x_max();
-        y_max = data->get_y_max();
+        y_max = 0;
+        v_foreach(Point, p, data->points())
+            if (p->is_active && p->y > y_max)
+                y_max = p->y;
     }
 #endif
     initialize_quick_phase_list();
@@ -207,9 +219,16 @@ PowderBook::PowderBook(wxWindow* parent, wxWindowID id)
     AddPage(PreparePeakPanel(), wxT("peak"), false, 3);
     AddPage(PrepareActionPanel(), wxT("action"), false, 4);
     AddPage(PrepareSizeStrainPanel(), wxT("size-strain"), false, 5);
+#if !STANDALONE_POWDIFPAT
     fill_forms();
+#endif
 
-    Connect(GetId(), wxEVT_COMMAND_LISTBOOK_PAGE_CHANGED,
+    wxColour c = GetToolBar()->GetBackgroundColour();
+    GetToolBar()->SetBackgroundColour(wxColour(max(0, c.Red() - 50),
+                                               max(0, c.Green() - 50),
+                                               c.Blue(), c.Alpha()));
+
+    Connect(GetId(), wxEVT_COMMAND_TOOLBOOK_PAGE_CHANGED,
             (wxObjectEventFunction) &PowderBook::OnPageChanged);
 }
 
@@ -261,21 +280,22 @@ wxPanel* PowderBook::PrepareIntroPanel()
     "is definitely more reliable.\n"
     "This is a stand-alone version of a dialog in fityk program.\n";
 #else
-    "Before you start, you should have:\n"
-    "  - data (powder diffraction pattern) loaded,\n"
-    "  - only interesting data range active,\n"
-    "  - baseline (background) either removed manually\n"
-    "    or modeled with e.g. polynomial.\n"
+    "Before you start:\n"
+    " - load a powder diffraction pattern,\n"
+    " - if you want to analyze only a part of the pattern, deactivate the rest"
     "\n"
-    "This tool can build a model for powder diffraction data, "
-    "The model has constrained position of peaks "
-    "and not constrained intensities. "
-    "Then you can fit the model to your data (all variables at the same time). "
-    "This type of refinement is known as Pawley method.\n"
+    " - either remove the background manually\n"
+    "   or add a polynomial (or other function) to model it.\n"
     "\n"
-    "The CCTBX library have been used to generate space group data.\n"
+    "This tool constructs a model for powder diffraction data. "
+    "The model has constrained positions of peaks "
+    "and unconstrained intensities. "
+    "Fitting the model to the data optimizes all parameters at the same time. "
+    "This type of refinement is known as the Pawley method.\n"
     "\n"
-    "This tool is new in fityk 0.9.3. The next release will contain more features.\n";
+    "Space group data have been generated using the CCTBX library.\n"
+    "\n"
+    "The size-strain analysis is not implemented yet.\n";
 #endif
 
     wxPanel *panel = new wxPanel(this);
@@ -303,7 +323,7 @@ wxPanel* PowderBook::PrepareIntroPanel()
     range_sizer->Add(range_to, wxSizerFlags().Center().Border());
     intro_sizer->Add(range_sizer);
     Connect(file_picker->GetId(), wxEVT_COMMAND_FILEPICKER_CHANGED,
-            wxCommandEventHandler(PowderBook::OnFilePicked));
+            (wxObjectEventFunction) &PowderBook::OnFilePicked);
 #endif
 
     wxStaticBoxSizer *legend_box = new wxStaticBoxSizer(wxHORIZONTAL,
@@ -311,7 +331,7 @@ wxPanel* PowderBook::PrepareIntroPanel()
     wxFlexGridSizer *legend = new wxFlexGridSizer(2, 5, 5);
     legend->Add(new wxStaticBitmap(panel, -1, wxBitmap(get_lock_xpm())),
                 wxSizerFlags().Center().Right());
-    legend->Add(new wxStaticText(panel, -1, wxT("constant parameter")),
+    legend->Add(new wxStaticText(panel, -1, wxT("parameter is known")),
                 wxSizerFlags().Center().Left());
     legend->Add(new wxStaticBitmap(panel, -1, wxBitmap(get_lock_open_xpm())),
                 wxSizerFlags().Center().Right());
@@ -346,7 +366,7 @@ wxPanel* PowderBook::PrepareInstrumentPanel()
     xaxis_choices.Add(wxT("2\u03B8")); //\u03B8 = theta
     xaxis_choices.Add(wxT("Q"));
     xaxis_choices.Add(wxT("d"));
-    xaxis_rb = new wxRadioBox(panel, -1, wxT("data x axis"),
+    xaxis_rb = new wxRadioBox(panel, -1, wxT("x axis"),
                                           wxDefaultPosition, wxDefaultSize,
                                           xaxis_choices, 3);
     // \u03C0 = pi, \u03BB = lambda
@@ -368,6 +388,9 @@ wxPanel* PowderBook::PrepareInstrumentPanel()
         anode_lb->Append(pchar2wx(i->name) + wxT(" A1"));
         anode_lb->Append(pchar2wx(i->name) + wxT(" A12"));
     }
+#ifdef __WXMAC__
+    anode_lb->EnsureVisible(0);
+#endif
     wave_sizer->Add(anode_lb, wxSizerFlags(1).Border().Expand());
 
     wxSizer *lambda_sizer = new wxFlexGridSizer(2, 5, 5);
@@ -449,8 +472,7 @@ PlotWithLines::PlotWithLines(wxWindow* parent, PhasePanel* phase_panel,
 double get_max_intensity(const vector<PlanesWithSameD>& bp)
 {
     double max_intensity = 0.;
-    for (vector<PlanesWithSameD>::const_iterator i = bp.begin();
-                                                        i != bp.end(); ++i)
+    v_foreach (PlanesWithSameD, i, bp)
         if (i->intensity > max_intensity)
             max_intensity = i->intensity;
     return max_intensity;
@@ -481,8 +503,7 @@ void PlotWithLines::draw(wxDC &dc, bool)
     double h_mult = 0;
     if (max_intensity > 0)
         h_mult = powder_book_->y_max / max_intensity;
-    for (vector<PlanesWithSameD>::const_iterator i = bp.begin();
-                                                        i != bp.end(); ++i) {
+    v_foreach (PlanesWithSameD, i, bp) {
         if (!i->enabled)
             continue;
         bool is_selected = (i - bp.begin() == selected);
@@ -520,6 +541,8 @@ void PlotWithLines::draw_active_data(wxDC& dc)
     if (!powder_book_->data)
         return;
     int n = powder_book_->data->get_n();
+    if (n == 0)
+        return;
     wxPoint *points = new wxPoint[n];
     for (int i = 0; i != n; ++i) {
         points[i].x = getX(powder_book_->data->get_x(i));
@@ -651,7 +674,7 @@ PhasePanel::PhasePanel(wxNotebook *parent, PowderBook *powder_book_)
 
 void PhasePanel::OnSpaceGroupButton(wxCommandEvent& event)
 {
-    SpaceGroupChooser dlg(this);
+    SpaceGroupChooser dlg(this, sg_tc->GetValue());
     if (dlg.ShowModal() == wxID_OK)
         sg_tc->SetValue(dlg.get_value());
     OnSpaceGroupChanged(event);
@@ -684,8 +707,7 @@ void PhasePanel::OnAddToQLButton(wxCommandEvent&)
     cel.alpha = par_alpha->get_value();
     cel.beta = par_beta->get_value();
     cel.gamma = par_gamma->get_value();
-    for (vector<Atom>::const_iterator i = cr_.atoms.begin();
-                                                i != cr_.atoms.end(); ++i) {
+    v_foreach (Atom, i, cr_.atoms) {
         t_pse const* pse = find_in_pse(i->symbol);
         assert (pse != NULL);
         AtomInCell aic;
@@ -730,12 +752,13 @@ void PhasePanel::OnSpaceGroupChanged(wxCommandEvent&)
     string text = wx2s(sg_tc->GetValue());
     const SpaceGroupSetting *sgs = parse_any_sg_symbol(text.c_str());
     cr_.set_space_group(sgs);
-    change_space_group();
+    update_space_group_ui();
+    update_miller_indices(true);
     powder_book->deselect_phase_quick_list();
     sample_plot_->refresh();
 }
 
-void PhasePanel::change_space_group()
+void PhasePanel::update_space_group_ui()
 {
     if (cr_.sgs == NULL) {
         sg_tc->Clear();
@@ -752,7 +775,6 @@ void PhasePanel::change_space_group()
                                         get_sg_order(cr_.sg_ops)));
     enable_parameter_fields();
     update_disabled_parameters();
-    update_miller_indices();
 }
 
 void PhasePanel::enable_parameter_fields()
@@ -768,29 +790,37 @@ void PhasePanel::enable_parameter_fields()
         case MonoclinicSystem:
             par_b->Enable(true);
             par_c->Enable(true);
-            par_alpha->Enable(true);
-            par_beta->Enable(false);
-            par_beta->set_value(90.);
+            par_beta->Enable(true);
+            par_alpha->Enable(false);
+            par_alpha->set_value(90.);
             par_gamma->Enable(false);
             par_gamma->set_value(90.);
             break;
         case OrthorhombicSystem:
-            par_b->Enable(false);
-            par_c->Enable(true);
-            set_ortho_angles();
-            break;
-        case TetragonalSystem:
             par_b->Enable(true);
             par_c->Enable(true);
             set_ortho_angles();
             break;
-        case TrigonalSystem:
+        case TetragonalSystem:
+            par_b->Enable(false);
+            par_c->Enable(true);
+            set_ortho_angles();
+            break;
+            /*
+        Crystals in the trigonal crystal system are either in the rhombohedral
+        lattice system or in the hexagonal lattice system.
+        Those in the rhombohedral space groups are described either using
+        the hexagonal basis or the rhombohedral basis.
+        The latter is not supported yet.
+        case RhombohedralSystem:
             par_b->Enable(false);
             par_c->Enable(false);
             par_alpha->Enable(true);
             par_beta->Enable(false);
             par_gamma->Enable(false);
             break;
+            */
+        case TrigonalSystem:
         case HexagonalSystem:
             par_b->Enable(false);
             par_c->Enable(true);
@@ -826,7 +856,7 @@ void PhasePanel::OnParameterChanging(wxCommandEvent&)
 
 void PhasePanel::OnParameterChanged(wxCommandEvent&)
 {
-    update_miller_indices();
+    update_miller_indices(false);
     sample_plot_->refresh();
 }
 
@@ -846,8 +876,7 @@ wxString make_info_string_for_line(const PlanesWithSameD& bp,
     wxString info = wxT("line ");
     wxString mult_str = wxT("multiplicity: ");
     wxString sfac_str = wxT("|F(hkl)|: ");
-    for (vector<Plane>::const_iterator i = bp.planes.begin();
-                                              i != bp.planes.end(); ++i) {
+    v_foreach (Plane, i, bp.planes) {
         if (i != bp.planes.begin()) {
             info += wxT(", ");
             mult_str += wxT(", ");
@@ -874,7 +903,7 @@ wxString make_info_string_for_line(const PlanesWithSameD& bp,
 wxString make_info_string_for_atoms(const vector<Atom>& atoms, int error_line)
 {
     wxString info = wxT("In unit cell:");
-    for (vector<Atom>::const_iterator i = atoms.begin(); i != atoms.end(); ++i){
+    v_foreach (Atom, i, atoms) {
         info += wxString::Format(wxT(" %d %s "), (int) i->pos.size(),
                                                  pchar2wx(i->symbol).c_str());
     }
@@ -949,13 +978,15 @@ void PhasePanel::update_disabled_parameters()
     if (!par_c->IsEnabled())
         par_c->set_string(par_a->get_string());
 
-    if (cr_.xs() == TrigonalSystem) {
+    /*
+    if (cr_.xs() == RhombohedralSystem) {
         par_beta->set_string(par_alpha->get_string());
         par_gamma->set_string(par_alpha->get_string());
     }
+    */
 }
 
-void PhasePanel::update_miller_indices()
+void PhasePanel::update_miller_indices(bool sg_changed)
 {
     double a = par_a->get_value();
     double b = par_b->get_value();
@@ -968,25 +999,34 @@ void PhasePanel::update_miller_indices()
     double min_d = powder_book->get_min_d();
     RadiationType radiation = powder_book->get_radiation_type();
 
-    hkl_list->Clear();
     if (a <= 0 || b <= 0 || c <= 0 || alpha <= 0 || beta <= 0 || gamma <= 0
             || min_d <= 0) {
+        hkl_list->Clear();
         return;
     }
-    cr_.set_unit_cell(a, b, c, alpha, beta, gamma);
 
-    cr_.generate_reflections(min_d);
-    cr_.update_intensities(radiation, lambda);
+    if (sg_changed || cr_.uc == NULL ||
+            cr_.uc->a != a || cr_.uc->b != b || cr_.uc->c != c ||
+            cr_.uc->alpha != alpha || cr_.uc->beta != beta ||
+            cr_.uc->gamma != gamma || cr_.old_min_d != min_d) {
+        cr_.set_unit_cell(new UnitCell(a, b, c, alpha, beta, gamma));
+        cr_.generate_reflections(min_d);
 
-    for (vector<PlanesWithSameD>::iterator i = cr_.bp.begin();
-                                                i != cr_.bp.end(); ++i) {
-        i->enabled = powder_book->is_d_active(i->d);
-        char a = (i->planes.size() == 1 ? ' ' : '*');
-        Miller const& m = i->planes[0];
-        hkl_list->Append(wxString::Format(wxT("(%d,%d,%d)%c  d=%g"),
-                                          m.h, m.k, m.l, a, i->d));
-        hkl_list->Check(hkl_list->GetCount() - 1, i->enabled);
+        hkl_list->Clear();
+        vm_foreach (PlanesWithSameD, i, cr_.bp) {
+            i->enabled = powder_book->is_d_active(i->d);
+            char a = (i->planes.size() == 1 ? ' ' : '*');
+            Miller const& m = i->planes[0];
+            hkl_list->Append(wxString::Format(wxT("(%d,%d,%d)%c  d=%g"),
+                                              m.h, m.k, m.l, a, i->d));
+            hkl_list->Check(hkl_list->GetCount() - 1, i->enabled);
+        }
+#ifdef __WXMAC__
+        hkl_list->EnsureVisible(0);
+#endif
     }
+
+    cr_.update_intensities(radiation, lambda);
 }
 
 void PhasePanel::set_phase(string const& name, CelFile const& cel)
@@ -994,7 +1034,6 @@ void PhasePanel::set_phase(string const& name, CelFile const& cel)
     name_tc->ChangeValue(s2wx(name));
     powder_book->update_phase_labels(this);
     cr_.set_space_group(cel.sgs);
-    change_space_group();
     par_a->set_string(wxString::Format(wxT("%g"), (cel.a)));
     par_b->set_string(wxString::Format(wxT("%g"), (cel.b)));
     par_c->set_string(wxString::Format(wxT("%g"), (cel.c)));
@@ -1002,8 +1041,7 @@ void PhasePanel::set_phase(string const& name, CelFile const& cel)
     par_beta->set_string(wxString::Format(wxT("%g"), (cel.beta)));
     par_gamma->set_string(wxString::Format(wxT("%g"), (cel.gamma)));
     wxString atoms_str;
-    for (vector<AtomInCell>::const_iterator i = cel.atoms.begin();
-            i != cel.atoms.end(); ++i) {
+    v_foreach (AtomInCell, i, cel.atoms) {
         t_pse const* pse = find_Z_in_pse(i->Z);
         if (pse == NULL)
             continue;
@@ -1012,12 +1050,14 @@ void PhasePanel::set_phase(string const& name, CelFile const& cel)
                                       i->x, i->y, i->z);
     }
     atoms_tc->ChangeValue(atoms_str);
+    update_space_group_ui();
+
     cr_.atoms.clear();
     line_with_error_ = parse_atoms((const char*) atoms_str.mb_str(), cr_);
     wxString info = make_info_string_for_atoms(cr_.atoms, line_with_error_);
     info_tc->SetValue(info);
     atoms_show_help_ = false;
-    update_miller_indices();
+    update_miller_indices(true);
 }
 
 wxPanel* PowderBook::PrepareSamplePanel()
@@ -1029,10 +1069,13 @@ wxPanel* PowderBook::PrepareSamplePanel()
     saved_phase_lb = new wxListBox(panel, -1, wxDefaultPosition, wxDefaultSize,
                                    0, NULL, wxLB_SINGLE|wxLB_SORT);
 
-    for (map<string, CelFile>::const_iterator i
-              = quick_phase_list.begin(); i != quick_phase_list.end(); ++i)
+    for (map<string, CelFile>::const_iterator i = quick_phase_list.begin();
+                                            i != quick_phase_list.end(); ++i)
         saved_phase_lb->Append(s2wx(i->first));
 
+#ifdef __WXMAC__
+    saved_phase_lb->EnsureVisible(0);
+#endif
     left_sizer->Add(saved_phase_lb, wxSizerFlags(1).Border().Expand());
     wxButton *s_remove_btn = new wxButton(panel, wxID_REMOVE);
     left_sizer->Add(s_remove_btn,
@@ -1149,60 +1192,87 @@ wxString PowderBook::get_peak_name() const
     return split_cb->GetValue() ? wxT("Split") + s : s;
 }
 
+void PowderBook::set_peak_name(const string& name)
+{
+    string basename;
+    if (startswith(name, "Split")) {
+        split_cb->SetValue(true);
+        basename = name.substr(5);
+    }
+    else
+        basename = name;
+
+    int sel = 0;
+    if (basename == "Gaussian")
+        sel = 0;
+    else if (basename == "Lorentzian")
+        sel = 1;
+    else if (basename == "Pearson7")
+        sel = 2;
+    else if (basename == "PseudoVoigt")
+        sel = 3;
+    else if (basename == "Voigt")
+        sel = 4;
+    peak_rb->SetSelection(sel);
+}
+
+#if !STANDALONE_POWDIFPAT
+static
+bool has_old_variables()
+{
+    v_foreach (Variable*, i, ftk->variables())
+        if (startswith((*i)->name, "pd"))
+            return true;
+    return false;
+}
+#endif
+
 wxPanel* PowderBook::PrepareActionPanel()
 {
     wxPanel *panel = new wxPanel(this);
+#if !STANDALONE_POWDIFPAT
     wxSizer *sizer = new wxBoxSizer(wxVERTICAL);
 
-    wxStaticBoxSizer *action_del_sizer;
-    action_del_sizer = new wxStaticBoxSizer(wxHORIZONTAL, panel,
-                                            wxT("Delete the old XRPD model"));
-    action_del_txt = new wxTextCtrl(panel, -1,
-                                    wxT("delete %pd*, $pd*"),
-                                    wxDefaultPosition, wxDefaultSize,
-                                    wxTE_READONLY);
-    action_del_sizer->Add(action_del_txt, wxSizerFlags(1).Border());
-    wxButton *action_del_btn = new wxButton(panel, wxID_APPLY);
-    action_del_sizer->Add(action_del_btn, wxSizerFlags().Border());
-
-    vector<Variable*> const& vv = ftk->get_variables();
-    bool has_old_model = false;
-    for (vector<Variable*>::const_iterator i = vv.begin(); i != vv.end(); ++i){
-        if (startswith((*i)->name, "pd")) {
-            has_old_model = true;
-            break;
-        }
-    }
-    action_del_txt->Enable(has_old_model);
-    action_del_btn->Enable(has_old_model);
-    action_del_sizer->GetStaticBox()->Enable(has_old_model);
-    sizer->Add(action_del_sizer, wxSizerFlags().Expand().Border());
-
-    wxStaticBoxSizer *a_set_sizer = new wxStaticBoxSizer(wxVERTICAL, panel,
-                                                   wxT("Set the XRPD model"));
+    sizer->Add(new wxStaticText(panel, -1, wxT("Script that prepares model:")),
+               wxSizerFlags().Border());
     action_txt = new wxTextCtrl(panel, -1, wxEmptyString,
                                 wxDefaultPosition, wxSize(-1, 200),
                                 wxTE_RICH|wxTE_READONLY|wxTE_MULTILINE);
-    //action_txt->SetBackgroundColour(GetBackgroundColour());
-    a_set_sizer->Add(action_txt, wxSizerFlags(1).Expand().Border());
-    a_set_sizer->Add(new wxStaticText(panel, -1,
-     wxT("Press OK to execute the script above that prepares the XRPD model.")
-     wxT("\nOK closes this window.")
-     wxT(" If the initial model is good, fit it to the data.")
-     wxT("\nThis tool can be reopened at any time to tune the model.")),
-                     wxSizerFlags().Border());
-    wxButton *ok_btn = new wxButton(panel, wxID_OK);
-    a_set_sizer->Add(ok_btn, wxSizerFlags().Right().Border(wxLEFT|wxRIGHT));
+    sizer->Add(action_txt, wxSizerFlags(1).Expand().Border(wxLEFT|wxRIGHT));
 
-    sizer->Add(a_set_sizer, wxSizerFlags(1).Expand().Border());
+    // apparently Mac/Carbon does not like multi-line static texts
+    wxStaticText *text = new wxStaticText(panel, -1,
+     wxT("Press OK to execute the script above and close this window.")
+#ifndef __WXMAC__
+     wxT("\nIf the initial model is good, fit it to the data.")
+     wxT("\nOtherwise, reopen this window and correct the model.")
+#endif
+     );
+    wxFont font = text->GetFont();
+    font.SetPointSize(font.GetPointSize() - 1);
+    text->SetFont(font);
+    sizer->Add(text, wxSizerFlags().Border());
+
+    wxBoxSizer *btn_sizer = new wxBoxSizer(wxHORIZONTAL);
+    wxButton *del_btn = new wxButton(panel, -1, wxT("Delete old model"));
+    del_btn->Enable(has_old_variables());
+    del_btn->SetToolTip(wxT("delete %pd*, $pd*"));
+    btn_sizer->Add(del_btn, wxSizerFlags().Border());
+    btn_sizer->AddStretchSpacer();
+    wxButton *cancel_btn = new wxButton(panel, wxID_CANCEL);
+    btn_sizer->Add(cancel_btn, wxSizerFlags().Border());
+    wxButton *ok_btn = new wxButton(panel, wxID_OK);
+    btn_sizer->Add(ok_btn, wxSizerFlags().Border());
+    sizer->Add(btn_sizer, wxSizerFlags().Expand());
 
     panel->SetSizerAndFit(sizer);
 
-    Connect(action_del_btn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
+    Connect(del_btn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
             wxCommandEventHandler(PowderBook::OnDelButton));
     Connect(ok_btn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
             wxCommandEventHandler(PowderBook::OnOk));
 
+#endif //!STANDALONE_POWDIFPAT
     return panel;
 }
 
@@ -1220,20 +1290,21 @@ wxString hkl2wxstr(const Miller& hkl)
                       hkl.l > -10 && hkl.l < 10);
     if (hkl.h < 0)
         s += wxT("m");
-    s += wxString::Format(wxT("%d"), hkl.h);
+    s += s2wx(S(abs(hkl.h)));
     if (separate)
         s += wxT("_");
     if (hkl.k < 0)
         s += wxT("m");
-    s += wxString::Format(wxT("%d"), hkl.k);
+    s += s2wx(S(abs(hkl.k)));
     if (separate)
         s += wxT("_");
     if (hkl.l < 0)
         s += wxT("m");
-    s += wxString::Format(wxT("%d"), hkl.l);
+    s += s2wx(S(abs(hkl.l)));
     return s;
 }
 
+#if !STANDALONE_POWDIFPAT
 // All variable and function names have "pd" prefix.
 // Each name is composed from a few parts.
 // One example: "$pd1b_c201":
@@ -1248,6 +1319,8 @@ wxString hkl2wxstr(const Miller& hkl)
 wxString PowderBook::prepare_commands()
 {
     wxString s;
+    if (has_old_variables())
+        s = wxT("delete %pd*, $pd* # delete old model\n");
 
     wxString ds_pref;
     if (ftk->get_dm_count() > 1) {
@@ -1278,14 +1351,14 @@ wxString PowderBook::prepare_commands()
         ++lambda_symbol;
 
     // corrections
-    if (UdfContainer::get_udf("PdXcorr") != NULL)
+    if (ftk->get_tpm()->get_tp("PdXcorr") != NULL)
         s += wxT("undefine PdXcorr\n");
     wxString xc_args, xc_vargs, xc_def;
-    wxString xc_formulas[] = { wxT("p1/tan(x)"),
-                               wxT("p2/sin(x)"),
-                               wxT("p3/tan(x/2)"),
-                               wxT("p4*sin(x)"),
-                               wxT("p5*cos(x/2)"),
+    wxString xc_formulas[] = { wxT("p1/tan(x*pi/180)"),
+                               wxT("p2/sin(x*pi/180)"),
+                               wxT("p3/tan(x/2*pi/180)"),
+                               wxT("p4*sin(x*pi/180)"),
+                               wxT("p5*cos(x/2*pi/180)"),
                                wxT("p6") };
     for (int i = 0; i != (int) corr_ctrl.size(); ++i) {
         if (corr_ctrl[i]->is_nonzero()) {
@@ -1313,10 +1386,6 @@ wxString PowderBook::prepare_commands()
         s += ds_pref + wxT("Z = %pd_xcorr\n");
     }
 
-    double max_intensity = get_max_intensity(
-                                        get_phase_panel(0)->get_crystal().bp);
-    double h_mult = y_max / (max_intensity > 0 ? max_intensity : 2);
-
     bool has_u = par_u->is_nonzero();
     bool has_v = par_v->is_nonzero();
     bool has_w = par_w->is_nonzero();
@@ -1353,6 +1422,10 @@ wxString PowderBook::prepare_commands()
     for (int i = 0; i < (int) sample_nb->GetPageCount() - 1; ++i) {
         const PhasePanel* p = get_phase_panel(i);
         const Crystal& cr = p->get_crystal();
+
+        double max_intensity = get_max_intensity(cr.bp);
+        double h_mult = y_max / (max_intensity > 0 ? max_intensity : 2);
+
         s += wxT("\n# ") + p->name_tc->GetValue() + wxT("\n");
         wxString pre = wxString::Format(wxT("$pd%d_"), i);
         s += pre + wxT("a = ") + get_var(p->par_a) + wxT("\n");
@@ -1368,11 +1441,18 @@ wxString PowderBook::prepare_commands()
         if (p->par_gamma->IsEnabled())
             s += pre + wxT("gamma = ") + get_var(p->par_gamma, d2r) + wxT("\n");
 
+        if (cr.xs() == TriclinicSystem) {
+            wxString v_str = wxT(":a*:b*:c *")
+                 wxT(" sqrt(1 - cos(:alpha)^2 - cos(:beta)^2 - cos(:gamma)^2")
+                 wxT(" + 2*cos(:alpha)*cos(:beta)*cos(:gamma))\n");
+            v_str.Replace(wxT(":"), pre);
+            s += pre + wxT("volume = ") + v_str;
+        }
+
         bool has_shape = peak_rb->GetSelection() >= 2;
         int shape_sel = shape_rb->GetSelection();
 
-        for (vector<PlanesWithSameD>::const_iterator j = cr.bp.begin();
-                                                     j != cr.bp.end(); ++j) {
+        v_foreach (PlanesWithSameD, j, cr.bp) {
             if (!j->enabled)
                 continue;
             const Miller& hkl = j->planes[0];
@@ -1381,11 +1461,12 @@ wxString PowderBook::prepare_commands()
             double h = h_mult * max(j->intensity, 1.);
             double lambda1 = get_lambda(0);
             double ctr = 180 / M_PI * 2 * asin(lambda1 / (2 * j->d));
-            s += hvar + wxString::Format(wxT(" = ~%.5g\n"), h);
+            s += wxT("\n") + hvar + wxString::Format(wxT(" = ~%.5g\n"), h);
 
             // we need to pre-define $pdXa_cHKL if wvar or svar depend on it
             wxString cvar_a = wxString::Format(wxT("$pd%da_c%s"),
                                                i, hkl_str.c_str());
+            wxString theta_rad = cvar_a + wxT("*pi/360");
             if (width_sel == 1 || (has_shape && shape_sel >= 1)) {
                 s += cvar_a + wxT(" = 0 # will be changed\n");
             }
@@ -1400,11 +1481,11 @@ wxString PowderBook::prepare_commands()
             else if (width_sel == 1 /*H2=Utan2T...*/) {
                 s +=  wvar + wxT(" = sqrt(");
                 if (has_u)
-                    s += wxT("$pd_u*tan(") + cvar_a + wxT("/2)^2");
+                    s += wxT("$pd_u*tan(") + theta_rad + wxT(")^2");
                 if (has_v) {
                     if (has_u)
                         s += wxT(" + ");
-                    s += wxT("$pd_v*tan(") + cvar_a + wxT("/2)");
+                    s += wxT("$pd_v*tan(") + theta_rad + wxT(")");
                 }
                 if (has_w) {
                     if (has_u || has_v)
@@ -1414,7 +1495,7 @@ wxString PowderBook::prepare_commands()
                 if (has_z) {
                     if (has_u || has_v || has_w)
                         s += wxT(" + ");
-                    s += wxT("$pd_z/cos(") + cvar_a + wxT("/2)^2");
+                    s += wxT("$pd_z/cos(") + theta_rad + wxT(")^2");
                 }
                 s += wxT(")\n");
             }
@@ -1449,22 +1530,51 @@ wxString PowderBook::prepare_commands()
                 wxString rd_str; // d^-1
                 switch (cr.xs()) {
                     case CubicSystem: // use only a
-                        rd_str.Printf(wxT("sqrt(%d)/$pd%d_a"),
-                                hkl.h*hkl.h + hkl.k*hkl.k + hkl.l*hkl.l, i);
+                        rd_str.Printf(wxT("sqrt(%d)/:a"),
+                                hkl.h*hkl.h + hkl.k*hkl.k + hkl.l*hkl.l);
                         break;
-                    case OrthorhombicSystem: // use a and c
-                        rd_str.Printf(wxT("sqrt(%d/$pd%d_a^2 + %d/$pd%d_c^2)"),
-                                  hkl.h*hkl.h + hkl.k*hkl.k, i, hkl.l*hkl.l, i);
+                    case TetragonalSystem: // use a and c
+                        rd_str.Printf(wxT("sqrt(%d/:a^2 + %d/:c^2)"),
+                                  hkl.h*hkl.h + hkl.k*hkl.k, hkl.l*hkl.l);
                         break;
-                    case TetragonalSystem: // use a, b, c
+                    //TODO rhombohedral basis are not supported
+                    case TrigonalSystem:
+                    case HexagonalSystem:
+                        rd_str.Printf(wxT("sqrt(%g/:a^2 + %d/:c^2)"),
+                          4./3.*(hkl.h*hkl.h + hkl.h*hkl.k + hkl.k*hkl.k),
+                          hkl.l*hkl.l);
+                        break;
+                    case OrthorhombicSystem: // use a, b, c
                         rd_str.Printf(
-                       wxT("sqrt(%d/$pd%d_a^2 + %d/$pd%d_b^2 + %d/$pd%d_c^2)"),
-                            hkl.h*hkl.h, i, hkl.k*hkl.k, i, hkl.l*hkl.l, i);
+                       wxT("sqrt(%d/:a^2 + %d/:b^2 + %d/:c^2)"),
+                            hkl.h*hkl.h, hkl.k*hkl.k, hkl.l*hkl.l);
                         break;
-                    //TODO
-                    default:
+                    case MonoclinicSystem:
+                        rd_str.Printf(
+                       wxT("(1/sin(:beta) * ")
+                       wxT("sqrt(%d/:a^2 + %d*sin(:beta)^2/:b^2 + %d/:c^2 ")
+                           wxT("- %d*cos(:beta)/(:a*:c)))"),
+                            hkl.h*hkl.h, hkl.k*hkl.k, hkl.l*hkl.l,
+                            2*hkl.h*hkl.l);
+                        break;
+                    case TriclinicSystem:
+                        rd_str.Printf(
+                wxT("(sqrt(")
+                wxT(" (%d*:b*:c*sin(:alpha))^2 + ")
+                wxT(" (%d*:a*:c*sin(:beta))^2 + ")
+                wxT(" (%d*:a*:b*sin(:gamma))^2 + ")
+                wxT("%d*:a*:b*:c^2*(cos(:alpha)*cos(:beta) - cos(:gamma)) + ")
+                wxT("%d*:a^2*:b*:c*(cos(:beta)*cos(:gamma) - cos(:alpha)) + ")
+                wxT("%d*:a*:b^2*:c*(cos(:alpha)*cos(:gamma) - cos(:beta)) ")
+                wxT(") / :volume)"),
+                            hkl.h, hkl.k, hkl.l,
+                            2*hkl.h*hkl.k, 2*hkl.k*hkl.l, 2*hkl.h*hkl.l);
+                        break;
+                    case UndefinedSystem:
+                        rd_str = wxT(" # undefined system # ");
                         break;
                 }
+                rd_str.Replace(wxT(":"), wxString::Format(wxT("$pd%d_"), i));
                 s += cvar;
                 if (xaxis_val == 0) // 2T
                     s += wxString::Format(
@@ -1522,7 +1632,7 @@ void var2lockctrl(const string& varname, LockableRealCtrl* ctrl, double mult=1.)
 // fills the form using assigned previously $pd* and %pd functions 
 void PowderBook::fill_forms()
 {
-    //instrument page
+    // instrument page
     for (size_t i = 0; i != lambda_ctrl.size(); ++i) {
         char wave = 'a' + i;
         var2lockctrl("pd_lambda_" + S(wave), lambda_ctrl[i]);
@@ -1533,16 +1643,14 @@ void PowderBook::fill_forms()
         var2lockctrl("pd_" + S(i+1), corr_ctrl[i]);
     }
 
+    int first_func = -1;
     // sample page
     for (size_t i = 0; ; ++i) {
         string pre = "pd" + S(i) + "_";
         int k = ftk->find_variable_nr(pre + "a");
         if (k == -1)
             break;
-        if (i == sample_nb->GetPageCount()) {
-            PhasePanel *page = new PhasePanel(sample_nb, this);
-            sample_nb->AddPage(page, wxT("unknown"));
-        }
+        assert(i == sample_nb->GetPageCount() - 1);
         PhasePanel* p = get_phase_panel(i);
         var2lockctrl(pre+"a", p->par_a);
         var2lockctrl(pre+"b", p->par_b);
@@ -1552,16 +1660,52 @@ void PowderBook::fill_forms()
         var2lockctrl(pre+"gamma", p->par_gamma);
 
         if (i < phase_desc.size()) {
-            p->name_tc->SetValue(phase_desc[i].name);
+            p->name_tc->ChangeValue(phase_desc[i].name);
+            update_phase_labels(p, i);
             p->sg_tc->ChangeValue(phase_desc[i].sg);
             wxCommandEvent dummy;
             p->OnSpaceGroupChanged(dummy);
             p->atoms_tc->SetValue(phase_desc[i].atoms);
-            //TODO: deselect peaks
+        }
+        Crystal& cr = p->get_crystal();
+        assert(cr.bp.size() == p->hkl_list->GetCount());
+        int n = 0;
+        vm_foreach (PlanesWithSameD, j, cr.bp) {
+            wxString fname = wxString::Format(wxT("pd%da_"), (int) i)
+                             + hkl2wxstr(j->planes[0]);
+            int nr = ftk->find_function_nr(wx2s(fname));
+            j->enabled = (nr != -1);
+            p->hkl_list->Check(n, j->enabled);
+            ++n;
+            if (j->enabled && first_func == -1)
+                first_func = nr;
         }
     }
 
     // peak page
+    if (first_func != -1) {
+        const Function *f = ftk->get_function(first_func);
+        set_peak_name(f->tp()->name);
+        int w = index_of_element(f->tp()->fargs, "hwhm");
+        if (w == -1)
+            w = index_of_element(f->tp()->fargs, "hwhm1");
+        if (w != -1) {
+            Variable* hwhm = ftk->get_variable(f->get_var_idx(w));
+            if (!hwhm->is_simple() || hwhm->name == "pd_w")
+                width_rb->SetSelection(1);
+        }
+        int sh = index_of_element(f->tp()->fargs, "shape");
+        if (sh == -1)
+            sh = index_of_element(f->tp()->fargs, "shape1");
+        if (sh != -1) {
+            Variable* shape = ftk->get_variable(f->get_var_idx(sh));
+            if (!shape->is_simple() || shape->name == "pd_a") {
+                string formula = shape->get_formula(ftk->parameters());
+                bool has_div = contains_element(formula, '/');
+                shape_rb->SetSelection(has_div ? 2 : 1);
+            }
+        }
+    }
     var2lockctrl("pd_u", par_u);
     var2lockctrl("pd_v", par_v);
     var2lockctrl("pd_w", par_w);
@@ -1569,8 +1713,8 @@ void PowderBook::fill_forms()
     var2lockctrl("pd_a", par_a);
     var2lockctrl("pd_b", par_b);
     var2lockctrl("pd_c", par_c);
-    // TODO: peak function, peak width, peak shape
 }
+#endif //!STANDALONE_POWDIFPAT
 
 wxPanel* PowderBook::PrepareSizeStrainPanel()
 {
@@ -1580,7 +1724,10 @@ wxPanel* PowderBook::PrepareSizeStrainPanel()
                                 wxDefaultPosition, wxDefaultSize,
                                 wxTE_RICH|wxTE_READONLY|wxTE_MULTILINE);
     txt->SetBackgroundColour(GetBackgroundColour());
-    txt->ChangeValue(wxT("It should be easy to put here simple size-strain analysis, like Williamson-Hall. But first we'd like to handle somehow instrumental broadening."));
+    txt->ChangeValue(
+            wxT("\nThe size-strain analysis is planned for ver. 1.1.0.")
+            wxT("\nCheck the NEWS file (linked from the Fityk website) ")
+            wxT("to see if it has been already implemented."));
     sizer->Add(txt, wxSizerFlags(1).Expand().Border());
 
     panel->SetSizerAndFit(sizer);
@@ -1731,17 +1878,19 @@ void PowderBook::OnLambdaChange(wxCommandEvent&)
     anode_lb->SetSelection(wxNOT_FOUND);
 }
 
-void PowderBook::OnPageChanged(wxListbookEvent& event)
+void PowderBook::OnPageChanged(wxBookCtrlEvent& event)
 {
     if (event.GetSelection() == 2) { // sample
         for (size_t i = 0; i != sample_nb->GetPageCount(); ++i) {
             PhasePanel* p = get_phase_panel(i);
-            p->update_miller_indices();
+            p->update_miller_indices(false);
             p->sample_plot_->refresh();
         }
     }
     else if (event.GetSelection() == 4) { // action
+#if !STANDALONE_POWDIFPAT
         action_txt->SetValue(prepare_commands());
+#endif
     }
 }
 
@@ -1794,11 +1943,15 @@ double PowderBook::d2x(double d) const
     return 0.;
 }
 
-double PowderBook::is_d_active(double d) const
+bool PowderBook::is_d_active(double d) const
 {
+#if !STANDALONE_POWDIFPAT
     double x = d2x(d);
     vector<Point>::const_iterator point = data->get_point_at(x);
     return point != data->points().end() && point->is_active;
+#else
+    return true;
+#endif
 }
 
 double PowderBook::get_min_d() const
@@ -1858,12 +2011,13 @@ void PowderBook::update_peak_parameters()
 
 // update wxNoteBook page labels: set the current page name to be the same
 // as the name in wxTextCtrl, make sure there is always one empty page.
-void PowderBook::update_phase_labels(PhasePanel* p)
+void PowderBook::update_phase_labels(PhasePanel* p, int active)
 {
     wxString name = p->name_tc->GetValue();
     bool valid = !name.empty();
     int last = sample_nb->GetPageCount() - 1;
-    int active = sample_nb->GetSelection();
+    if (active == -1)
+        active = sample_nb->GetSelection();
     wxString empty_label = wxT("+");
     bool last_empty = (sample_nb->GetPageText(last) == empty_label);
 
@@ -1886,10 +2040,10 @@ void PowderBook::update_phase_labels(PhasePanel* p)
     }
 }
 
+#if !STANDALONE_POWDIFPAT
 void PowderBook::OnDelButton(wxCommandEvent&)
 {
-    wxString script = action_del_txt->GetValue();
-    ftk->get_ui()->exec_string_as_script(script.mb_str());
+    ftk->exec("delete %pd*, $pd*");
 }
 
 void PowderBook::OnOk(wxCommandEvent&)
@@ -1897,11 +2051,12 @@ void PowderBook::OnOk(wxCommandEvent&)
     wxString script = action_txt->GetValue();
     ftk->get_ui()->exec_string_as_script(script.mb_str());
     save_phase_desc();
-#if !STANDALONE_POWDIFPAT
     wxDialog* dialog = static_cast<wxDialog*>(GetParent());
     dialog->EndModal(wxID_OK);
-#endif
+    // unlike exec(), exec_string_as_script() does not call after_cmd_updates()
+    frame->after_cmd_updates();
 }
+#endif
 
 void PowderBook::save_phase_desc()
 {
@@ -2025,7 +2180,7 @@ void App::OnAbout(wxCommandEvent&)
     wxString desc = wxT("Powder diffraction pattern generator.\n");
     adi.SetDescription(desc);
     adi.SetWebSite(wxT("http://www.unipress.waw.pl/fityk/powdifpat/"));
-    adi.SetCopyright(wxT("(C) 2008 - 2010 Marcin Wojdyr <wojdyr@gmail.com>"));
+    adi.SetCopyright(wxT("(C) 2008 - 2011 Marcin Wojdyr <wojdyr@gmail.com>"));
     wxAboutBox(adi);
 }
 
@@ -2041,7 +2196,11 @@ PowderDiffractionDlg::PowderDiffractionDlg(wxWindow* parent, wxWindowID id)
     sizer->Add(pb, wxSizerFlags(1).Expand());
     SetSizerAndFit(sizer);
     if (GetClientSize().GetHeight() < 440)
+#ifdef __WXMAC__
+        SetClientSize(-1, 520);
+#else
         SetClientSize(-1, 440);
+#endif
 }
 
 #endif
