@@ -1,6 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: pplot.h 589 2009-11-20 20:31:41Z wojdyr $
 
 #ifndef FITYK_WX_PPLOT_H_
 #define FITYK_WX_PPLOT_H_
@@ -24,27 +23,21 @@ public:
     void save_settings(wxConfigBase *cf) const;
     void read_settings(wxConfigBase *cf);
     void refresh_plots(bool now, WhichPlot which_plot);
-    void set_mouse_mode(MouseModeEnum m);
     bool is_background_white();
     std::vector<std::string> const& get_zoom_hist() const { return zoom_hist; }
     MainPlot const* get_plot() const { return plot; }
     MainPlot* get_plot() { return plot; }
-    BgManager* get_bg_manager();
-    std::vector<FPlot*> const get_visible_plots() const;
     AuxPlot* get_aux_plot(int n) const
                      { assert(n>=0 && n<2); return aux_plot[n]; }
     void show_aux(int n, bool show);
     bool aux_visible(int n) const;
-    void update_crosshair(int X, int Y);
+    void draw_vertical_lines(int X1, int X2, FPlot* skip);
 
-    bool crosshair_cursor;
 private:
     MainPlot *plot;
     ProportionalSplitter *aux_split;
     AuxPlot *aux_plot[2];
     std::vector<std::string> zoom_hist;
-
-    void do_draw_crosshair(int X, int Y);
 };
 
 #endif // FITYK_WX_PPLOT_H_
