@@ -1,6 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: setdlg.h 522 2009-07-27 12:25:56Z wojdyr $
 
 #ifndef FITYK__WX_SETDLG__H__
 #define FITYK__WX_SETDLG__H__
@@ -10,30 +9,23 @@
 #include <utility>
 #include "cmn.h" //s2wx, RealNumberCtrl
 
-class SpinCtrl;
-
 class SettingsDlg : public wxDialog
 {
 public:
-    typedef std::vector<std::pair<std::string, std::string> > pair_vec;
-    SettingsDlg(wxWindow* parent, const wxWindowID id);
-    void OnChangeButton(wxCommandEvent& event);
+    SettingsDlg(wxWindow* parent);
     void OnOK(wxCommandEvent& event);
-    pair_vec get_changed_items();
 private:
-    wxChoice *sigma_ch, *verbosity_ch, *export_f_ch, *nm_distrib;
+    wxChoice *sigma_ch, *nm_distrib;
     wxCheckBox *exit_cb;
-    SpinCtrl *delay_sp, *seed_sp, *mwssre_sp;
+    wxSpinCtrl *delay_sp, *seed_sp, *mwssre_sp, *verbosity_sp;
     RealNumberCtrl *cut_func, *eps_rc, *height_correction, *width_correction,
                    *domain_p, *lm_lambda_ini, *lm_lambda_up, *lm_lambda_down,
                    *lm_stop, *lm_max_lambda,
                    *nm_convergence, *nm_move_factor;
-    wxCheckBox *cancel_poos, *nm_move_all, *autoplot_cb;
-    wxTextCtrl *dir_ld_tc, *dir_xs_tc, *dir_ex_tc;
+    wxCheckBox *nm_move_all, *fit_replot_cb;
+    wxTextCtrl *format_tc;
 
-    void add_persistence_note(wxWindow *parent, wxSizer *sizer);
-
-    DECLARE_EVENT_TABLE()
+    void exec_set_command();
 };
 
 #endif
