@@ -1,9 +1,7 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: sidebar.cpp 743 2010-06-12 11:52:07Z wojdyr $
 
-/// In this file:
-///  SideBar class, the right hand sidebar in GUI
+/// class SideBar
 
 #include <wx/wxprec.h>
 #ifdef __BORLANDC__
@@ -21,6 +19,7 @@
 #include "gradient.h"
 #include "cmn.h" //SpinCtrl, ProportionalSplitter, change_color_dlg, ...
 #include "frame.h" //frame
+#include "pplot.h"
 #include "mplot.h"
 #include "../common.h" //vector4, join_vector, S(), ...
 #include "../ui.h"
@@ -182,15 +181,15 @@ SideBar::SideBar(wxWindow *parent, wxWindowID id)
     dpline_cb->SetToolTip(wxT("line between data points"));
     data_spin_sizer->Add(dpline_cb, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 5);
     // line between points
-    dpsigma_cb = new wxCheckBox(data_page, ID_DP_PS, wxT("s"));
-    dpsigma_cb->SetToolTip(wxT("show std. dev. of y (sigma)"));
+    dpsigma_cb = new wxCheckBox(data_page, ID_DP_PS, wxT("\u03C3")); // sigma
+    dpsigma_cb->SetToolTip(wxT("show std. dev. of y"));
     data_spin_sizer->Add(dpsigma_cb, 0, wxALIGN_CENTER_VERTICAL|wxLEFT, 5);
     // shift-up spin button
     data_spin_sizer->AddStretchSpacer();
     data_spin_sizer->Add(new wxStaticBitmap(data_page, -1,
                                             wxBitmap(shiftup_xpm)),
                          0, wxALIGN_CENTER_VERTICAL|wxLEFT, 5);
-    shiftup_sc = new SpinCtrl(data_page, ID_DP_SHIFTUP, 0, 0, 80, 40);
+    shiftup_sc = new SpinCtrl(data_page, ID_DP_SHIFTUP, 0, 0, 80, 50);
     shiftup_sc->SetToolTip(wxT("shift up (in % of plot height)"));
     data_spin_sizer->Add(shiftup_sc, 0);
     data_sizer->Add(data_spin_sizer, 0, wxEXPAND);
@@ -258,6 +257,12 @@ SideBar::SideBar(wxWindow *parent, wxWindowID id)
     func_page->SetSizerAndFit(func_sizer);
     nb->AddPage(func_page, wxT("functions"));
 
+    // "=W" and "=S" buttons look better with the same size as other buttons
+    wxSize size = FindWindow(ID_FP_COL)->GetSize();
+    func_buttons_sizer->SetItemMinSize((size_t) 0,
+                                        size.GetWidth(), size.GetHeight());
+    func_buttons_sizer->SetItemMinSize(1, size.GetWidth(), size.GetHeight());
+
     //-----  variables page  -----
     var_page = new wxPanel(nb, -1);
     wxBoxSizer *var_sizer = new wxBoxSizer(wxVERTICAL);
@@ -302,7 +307,7 @@ void SideBar::OnDataButtonRen (wxCommandEvent&)
                          wxT("Rename dataset"),
                          old_title);
     if (!s.IsEmpty() && s != old_title)
-        ftk->exec("set @" + S(n) + ".title = '" + wx2s(s) + "'");
+        ftk->exec("@" + S(n) + ".title = '" + wx2s(s) + "'");
 }
 
 void SideBar::delete_selected_items()
@@ -311,15 +316,23 @@ void SideBar::delete_selected_items()
     if (n < 0)
         return;
     string txt = wx2s(nb->GetPageText(n));
+    vector<string> elems;
     if (txt == "data") {
-        ftk->exec("delete " + join_vector(d->get_selected_data(), ", "));
+        elems = d->get_selected_data();
+    }
+    else if (txt == "functions") {
+        elems = get_selected_func();
+        vm_foreach (string, i, elems)
+            *i = "%" + *i;
+    }
+    else if (txt == "variables") {
+        elems = get_selected_vars();
+        vm_foreach (string, i, elems)
+            *i = "$" + *i;
     }
-    else if (txt == "functions")
-        ftk->exec("delete " + join_vector(get_selected_func(), ", "));
-    else if (txt == "variables")
-        ftk->exec("delete " + join_vector(get_selected_vars(), ", "));
     else
         assert(0);
+    ftk->exec("delete " + join_vector(elems, ", "));
 }
 
 void SideBar::OnDataButtonCopyF (wxCommandEvent&)
@@ -331,8 +344,8 @@ void SideBar::OnDataButtonCopyF (wxCommandEvent&)
     d->list->Select(n+1, true);
     d->list->Focus(n+1);
     string cmd = "@" + S(n+1) + ".F=copy(@" + S(n) + ".F)";
-    if (!ftk->get_model(n)->get_zz_names().empty()
-            || !ftk->get_model(n+1)->get_zz_names().empty())
+    if (!ftk->get_model(n)->get_zz().names.empty()
+            || !ftk->get_model(n+1)->get_zz().names.empty())
         cmd += "; @" + S(n+1) + ".Z=copy(@" + S(n) + ".Z)";
     if (ftk->find_function_nr("bg" + S(n)) != -1)
         cmd += "; %bg" + S(n+1) + "=copy(%bg" + S(n) + ")";
@@ -351,7 +364,7 @@ void SideBar::OnDataButtonCol (wxCommandEvent&)
         if (change_color_dlg(col)) {
             frame->get_main_plot()->set_data_color(n, col);
             update_lists();
-            frame->refresh_plots(false, kMainPlot);
+            frame->plot_pane()->refresh_plots(false, kMainPlot);
         }
     }
     else {//sel_size > 1
@@ -361,14 +374,21 @@ void SideBar::OnDataButtonCol (wxCommandEvent&)
             last_sel = i;
         wxColour first_col = frame->get_main_plot()->get_data_color(first_sel);
         wxColour last_col = frame->get_main_plot()->get_data_color(last_sel);
-        GradientDlgWithApply<SideBar> gd(this, -1, first_col, last_col,
-                                         this, &SideBar::OnDataColorsChanged);
+        GradientDlg gd(this, -1, first_col, last_col);
+        gd.FindWindow(wxID_APPLY)->Connect(wxEVT_COMMAND_BUTTON_CLICKED,
+                        wxCommandEventHandler(SideBar::OnDataColorsChanged),
+                        NULL, this);
         gd.ShowModal();
     }
 }
 
-void SideBar::OnDataColorsChanged(GradientDlg *gd)
+void SideBar::OnDataColorsChanged(wxCommandEvent& event)
 {
+    wxWindow* object = wxDynamicCast(event.GetEventObject(), wxWindow);
+    if (object == NULL || object->GetId() != wxID_APPLY)
+        return;
+    GradientDlg *gd = static_cast<GradientDlg*>(object->GetParent());
+
     vector<int> selected;
     for (int i = d->list->GetFirstSelected(), c = 0; i != -1;
                                         i = d->list->GetNextSelected(i), ++c) {
@@ -377,9 +397,7 @@ void SideBar::OnDataColorsChanged(GradientDlg *gd)
         frame->get_main_plot()->set_data_color(i, col);
     }
     update_lists();
-    frame->refresh_plots(false, kMainPlot);
-    for (vector<int>::const_iterator i = selected.begin();
-            i != selected.end(); ++i)
+    frame->plot_pane()->refresh_plots(false, kMainPlot);
     for (int i = 0; i != d->list->GetItemCount(); ++i)
         d->list->Select(i, contains_element(selected, i));
 }
@@ -387,12 +405,12 @@ void SideBar::OnDataColorsChanged(GradientDlg *gd)
 
 void SideBar::OnDataLookChanged (wxCommandEvent&)
 {
-    frame->refresh_plots(false, kMainPlot);
+    frame->plot_pane()->refresh_plots(false, kMainPlot);
 }
 
 void SideBar::OnDataShiftUpChanged (wxSpinEvent&)
 {
-    frame->refresh_plots(false, kMainPlot);
+    frame->plot_pane()->refresh_plots(false, kMainPlot);
 }
 
 void SideBar::OnDataPSizeChanged (wxSpinEvent& event)
@@ -402,19 +420,19 @@ void SideBar::OnDataPSizeChanged (wxSpinEvent& event)
     //    frame->get_main_plot()->set_data_point_size(i, event.GetPosition());
     // now it is set globally
     frame->get_main_plot()->set_data_point_size(0, event.GetPosition());
-    frame->refresh_plots(false, kMainPlot);
+    frame->plot_pane()->refresh_plots(false, kMainPlot);
 }
 
 void SideBar::OnDataPLineChanged (wxCommandEvent& event)
 {
     frame->get_main_plot()->set_data_with_line(0, event.IsChecked());
-    frame->refresh_plots(false, kMainPlot);
+    frame->plot_pane()->refresh_plots(false, kMainPlot);
 }
 
 void SideBar::OnDataPSChanged (wxCommandEvent& event)
 {
     frame->get_main_plot()->set_data_with_sigma(0, event.IsChecked());
-    frame->refresh_plots(false, kMainPlot);
+    frame->plot_pane()->refresh_plots(false, kMainPlot);
 }
 
 void SideBar::OnFuncFilterChanged (wxCommandEvent&)
@@ -425,10 +443,9 @@ void SideBar::OnFuncFilterChanged (wxCommandEvent&)
 void SideBar::OnFuncButtonNew (wxCommandEvent&)
 {
     string peak_type = frame->get_peak_type();
-    string formula = Function::get_formula(peak_type);
-    vector<string> varnames = Function::get_varnames_from_formula(formula);
+    const Tplate *tp = ftk->get_tpm()->get_tp(peak_type);
     string t = "%put_name_here = " + peak_type + "("
-                                      + join_vector(varnames, "= , ") + "= )";
+                                      + join_vector(tp->fargs, "= , ") + "= )";
     frame->edit_in_input(t);
 }
 
@@ -436,8 +453,8 @@ void SideBar::OnFuncButtonEdit (wxCommandEvent&)
 {
     if (!pp_func)
         return;
-    string t = pp_func->get_current_assignment(ftk->get_variables(),
-                                               ftk->get_parameters());
+    string t = pp_func->get_current_assignment(ftk->variables(),
+                                               ftk->parameters());
     frame->edit_in_input(t);
 }
 
@@ -449,7 +466,7 @@ void SideBar::OnFuncButtonChType (wxCommandEvent&)
 void SideBar::OnFuncButtonCol (wxCommandEvent&)
 {
     vector<int> const& ffi
-        = ftk->get_model(frame->get_focused_data_index())->get_ff_idx();
+        = ftk->get_model(frame->get_focused_data_index())->get_ff().idx;
     vector<int>::const_iterator in_ff = find(ffi.begin(), ffi.end(),
                                                          active_function);
     if (in_ff == ffi.end())
@@ -459,7 +476,7 @@ void SideBar::OnFuncButtonCol (wxCommandEvent&)
     if (change_color_dlg(col)) {
         frame->get_main_plot()->set_func_color(color_id, col);
         update_lists();
-        frame->refresh_plots(false, kMainPlot);
+        frame->plot_pane()->refresh_plots(false, kMainPlot);
     }
 }
 
@@ -471,10 +488,10 @@ void SideBar::OnVarButtonNew (wxCommandEvent&)
 void SideBar::OnVarButtonEdit (wxCommandEvent&)
 {
     int n = get_focused_var();
-    if (n < 0 || n >= size(ftk->get_variables()))
+    if (n < 0 || n >= size(ftk->variables()))
         return;
     Variable const* var = ftk->get_variable(n);
-    string t = var->xname + " = "+ var->get_formula(ftk->get_parameters());
+    string t = "$" + var->name + " = "+ var->get_formula(ftk->parameters());
     frame->edit_in_input(t);
 }
 
@@ -541,7 +558,7 @@ void SideBar::update_func_list(bool nondata_changed)
     Model const* filter_model = NULL;
     if (filter_ch->GetSelection() > 0)
         filter_model = ftk->get_model(filter_ch->GetSelection()-1);
-    int func_size = ftk->get_functions().size();
+    int func_size = ftk->functions().size();
     if (active_function == -1)
         active_function = func_size - 1;
     else {
@@ -558,20 +575,21 @@ void SideBar::update_func_list(bool nondata_changed)
 
     int pos = -1;
     for (int i = 0; i < func_size; ++i) {
-        if (filter_model && !contains_element(filter_model->get_ff_idx(), i)
-                           && !contains_element(filter_model->get_zz_idx(), i))
+        if (filter_model && !contains_element(filter_model->get_ff().idx, i)
+                           && !contains_element(filter_model->get_zz().idx, i))
             continue;
         if (i == active_function)
             pos = new_func_col_id.size();
         Function const* f = ftk->get_function(i);
         func_data.push_back(f->name);
-        func_data.push_back(f->type_name);
-        func_data.push_back(f->has_center() ? S(f->center()).c_str() : "-");
-        func_data.push_back(f->has_area() ? S(f->area()).c_str() : "-");
-        func_data.push_back(f->has_height() ? S(f->height()).c_str() : "-");
-        func_data.push_back(f->has_fwhm() ? S(f->fwhm()).c_str() : "-");
-        vector<int> const& ffi = model->get_ff_idx();
-        vector<int> const& zzi = model->get_zz_idx();
+        func_data.push_back(f->tp()->name);
+        realt a;
+        func_data.push_back(f->get_center(&a) ? S(a) : S("-"));
+        func_data.push_back(f->get_area(&a)   ? S(a) : S("-"));
+        func_data.push_back(f->get_height(&a) ? S(a) : S("-"));
+        func_data.push_back(f->get_fwhm(&a)   ? S(a) : S("-"));
+        vector<int> const& ffi = model->get_ff().idx;
+        vector<int> const& zzi = model->get_zz().idx;
         vector<int>::const_iterator in_ff = find(ffi.begin(), ffi.end(), i);
         int color_id = -2;
         if (in_ff != ffi.end())
@@ -584,8 +602,7 @@ void SideBar::update_func_list(bool nondata_changed)
     if (nondata_changed || func_col_id != new_func_col_id) {
         func_col_id = new_func_col_id;
         func_images = new wxImageList(16, 16);
-        for (vector<int>::const_iterator i = func_col_id.begin();
-                                                i != func_col_id.end(); ++i) {
+        v_foreach (int, i, func_col_id) {
             if (*i == -2)
                 func_images->Add(wxBitmap(unused_xpm));
             else if (*i == -1)
@@ -605,15 +622,13 @@ void SideBar::update_var_list()
 {
     vector<string> var_data;
     //  count references first
-    vector<Variable*> const& variables = ftk->get_variables();
+    vector<Variable*> const& variables = ftk->variables();
     vector<int> var_vrefs(variables.size(), 0), var_frefs(variables.size(), 0);
-    for (vector<Variable*>::const_iterator i = variables.begin();
-                                                i != variables.end(); ++i) {
+    v_foreach (Variable*, i, variables) {
         for (int j = 0; j != (*i)->get_vars_count(); ++j)
             var_vrefs[(*i)->get_var_idx(j)]++;
     }
-    for (vector<Function*>::const_iterator i = ftk->get_functions().begin();
-                                         i != ftk->get_functions().end(); ++i) {
+    v_foreach (Function*, i, ftk->functions()) {
         for (int j = 0; j != (*i)->get_vars_count(); ++j)
             var_frefs[(*i)->get_var_idx(j)]++;
     }
@@ -625,7 +640,7 @@ void SideBar::update_var_list()
                       + S(v->get_vars_count());
         var_data.push_back(refs); //refs
         var_data.push_back(S(v->get_value())); //value
-        var_data.push_back(v->get_formula(ftk->get_parameters()));  //formula
+        var_data.push_back(v->get_formula(ftk->parameters()));  //formula
     }
     v->list->populate(var_data);
 }
@@ -649,7 +664,7 @@ int SideBar::get_focused_data() const
 
 int SideBar::get_focused_var() const
 {
-    if (ftk->get_variables().empty())
+    if (ftk->variables().empty())
         return -1;
     else {
         int n = v->list->GetFocusedItem();
@@ -706,20 +721,18 @@ vector<int> SideBar::get_ordered_dataset_numbers()
     return ordered;
 }
 
-string SideBar::get_plot_in_datasets()
+string SideBar::get_sel_datasets_as_string()
 {
     if (ftk->get_dm_count() == 1)
         return "";
-    int focused = get_focused_data();
-    string s = " in @" + S(focused);
     if (data_look->GetSelection() == 0) // all datasets
-        s += ", @*";
-    else {
-        for (int i = d->list->GetFirstSelected(); i != -1;
+        return " @*";
+    string s;
+    for (int i = d->list->GetFirstSelected(); i != -1;
                                              i = d->list->GetNextSelected(i))
-            if (i != focused)
-                s += ", @" + S(i);
-    }
+        s += " @" + S(i);
+    if (s.empty())
+        s = " @" + S(get_focused_data());
     return s;
 }
 
@@ -757,7 +770,7 @@ void SideBar::do_activate_function()
         active_function_name = ftk->get_function(active_function)->name;
     else
         active_function_name = "";
-    frame->refresh_plots(false, kMainPlot);
+    frame->plot_pane()->refresh_plots(false, kMainPlot);
     update_func_inf();
     update_param_panel();
 }
@@ -785,14 +798,14 @@ void SideBar::OnDataFocusChanged(wxListEvent &)
         return;
     int length = ftk->get_dm_count();
     if (length > 1)
-        frame->refresh_plots(false, kAllPlots);
+        frame->plot_pane()->refresh_plots(false, kAllPlots);
     update_data_inf();
 }
 
 void SideBar::OnDataSelectionChanged(wxListEvent &/*event*/)
 {
     if (data_look->GetSelection() != 0) // ! all datasets
-        frame->refresh_plots(false, kAllPlots);
+        frame->plot_pane()->refresh_plots(false, kAllPlots);
     //bool selected= (event.GetEventType() == wxEVT_COMMAND_LIST_ITEM_SELECTED);
     //assert (selected
     //        || event.GetEventType() == wxEVT_COMMAND_LIST_ITEM_DESELECTED);
@@ -822,11 +835,10 @@ void SideBar::update_func_buttons()
     func_page->FindWindow(ID_FP_COL)->Enable(sel_f > 0);
 
     bool has_hwhm = false, has_shape = false;
-    for (vector<Function*>::const_iterator i = ftk->get_functions().begin();
-                                         i != ftk->get_functions().end(); ++i) {
-        if ((*i)->get_param_nr_nothrow("hwhm") != -1)
+    v_foreach (Function*, i, ftk->functions()) {
+        if (contains_element((*i)->tp()->fargs, "hwhm"))
             has_hwhm = true;
-        if ((*i)->get_param_nr_nothrow("shape") != -1)
+        if (contains_element((*i)->tp()->fargs, "shape"))
             has_shape = true;
     }
     func_page->FindWindow(ID_FP_HWHM)->Enable(has_hwhm);
@@ -882,23 +894,26 @@ void SideBar::update_func_inf()
     if (active_function < 0)
         return;
     Function const* func = ftk->get_function(active_function);
-    if (func->has_center())
-        inf->AppendText(wxString::Format(wxT("Center: %.10g"), func->center()));
-    if (func->has_area())
-        inf->AppendText(wxT("\nArea: ") + s2wx(S(func->area())));
-    if (func->has_height())
-        inf->AppendText(wxT("\nHeight: ") + s2wx(S(func->height())));
-    if (func->has_fwhm())
-        inf->AppendText(wxT("\nFWHM: ") + s2wx(S(func->fwhm())));
-    if (func->has_iwidth())
-        inf->AppendText(wxT("\nInt. Width: ") + s2wx(S(func->iwidth())));
-    if (func->has_other_props())
-        inf->AppendText(wxT("\n") + s2wx(func->other_props_str()));
+    realt a;
+    if (func->get_center(&a))
+        inf->AppendText(wxT("Center: ")
+                   + s2wx(format1<realt, 30>("%.10"REALT_LENGTH_MOD"g", a)));
+    if (func->get_area(&a))
+        inf->AppendText(wxT("\nArea: ") + s2wx(S(a)));
+    if (func->get_height(&a))
+        inf->AppendText(wxT("\nHeight: ") + s2wx(S(a)));
+    if (func->get_fwhm(&a))
+        inf->AppendText(wxT("\nFWHM: ") + s2wx(S(a)));
+    if (func->get_iwidth(&a))
+        inf->AppendText(wxT("\nInt. Width: ") + s2wx(S(a)));
+    v_foreach (string, i, func->get_other_prop_names())
+        inf->AppendText(s2wx("\n" + *i + ": " + S(func->get_other_prop(*i))));
+
     vector<string> in;
     for (int i = 0; i < ftk->get_dm_count(); ++i) {
-        if (contains_element(ftk->get_model(i)->get_ff_idx(), active_function))
+        if (contains_element(ftk->get_model(i)->get_ff().idx, active_function))
             in.push_back("@" + S(i) + ".F");
-        if (contains_element(ftk->get_model(i)->get_zz_idx(), active_function))
+        if (contains_element(ftk->get_model(i)->get_zz().idx, active_function))
             in.push_back("@" + S(i) + ".Z");
     }
     if (!in.empty())
@@ -914,7 +929,7 @@ void SideBar::update_var_inf()
     if (n < 0)
         return;
     Variable const* var = ftk->get_variable(n);
-    string t = var->xname + " = " + var->get_formula(ftk->get_parameters());
+    string t = "$" + var->name + " = " + var->get_formula(ftk->parameters());
     inf->AppendText(s2wx(t));
     vector<string> in = ftk->get_variable_references(var->name);
     if (!in.empty())
@@ -942,7 +957,7 @@ vector<string> SideBar::get_selected_func() const
     vector<string> dd;
     for (int i = f->list->GetFirstSelected(); i != -1;
                                             i = f->list->GetNextSelected(i))
-        dd.push_back("%" + wx2s(f->list->GetItemText(i)));
+        dd.push_back(wx2s(f->list->GetItemText(i)));
     //if (dd.empty() && f->list->GetItemCount() > 0) {
     //    int n = f->list->GetFocusedItem();
     //    dd.push_back(ftk->get_function(n == -1 ? 0 : n)->xname);
@@ -955,7 +970,7 @@ vector<string> SideBar::get_selected_vars() const
     vector<string> dd;
     for (int i = v->list->GetFirstSelected(); i != -1;
                                              i = v->list->GetNextSelected(i))
-        dd.push_back(ftk->get_variable(i)->xname);
+        dd.push_back(ftk->get_variable(i)->name);
     return dd;
 }
 
@@ -983,16 +998,19 @@ bool SideBar::find_value_of_param(string const& p, double* value)
 {
     if (active_function != -1) {
         Function const* f = ftk->get_function(active_function);
-        bool found = f->get_param_value_nothrow(p, *value);
-        if (found)
+        int idx = index_of_element(f->tp()->fargs, p);
+        if (idx != -1) {
+            *value = f->av()[idx];
             return true;
+        }
     }
 
-    vector<Function*> const& ff = ftk->get_functions();
-    for (vector<Function*>::const_iterator i = ff.begin(); i != ff.end(); ++i) {
-        bool found = (*i)->get_param_value_nothrow(p, *value);
-        if (found)
+    v_foreach (Function*, i, ftk->functions()) {
+        int idx = index_of_element((*i)->tp()->fargs, p);
+        if (idx != -1) {
+            *value = (*i)->av()[idx];
             return true;
+        }
     }
     return false;
 }
@@ -1000,49 +1018,64 @@ bool SideBar::find_value_of_param(string const& p, double* value)
 void SideBar::make_same_func_par(string const& p, bool checked)
 {
     string varname = "_" + p;
-    string cmd;
+    string val_str;
     if (checked) {
-        fp value = 0.;
+        double value = 0.;
         bool found = find_value_of_param(p, &value);
         if (!found)
             return;
 
-        cmd += "$" + varname + " = ~" + S(value);
-        for (int i = 0; i < ftk->get_dm_count(); ++i)
-            if (ftk->get_model(i)->get_ff_names().size() > 0)
-                cmd += "; @" + S(i) + ".F." + p + " = $" + varname;
+        ftk->exec("$" + varname + " = ~" + S(value));
+        val_str = "$" + varname;
     }
     else {
         int nr = ftk->find_variable_nr(varname);
         if (nr == -1)
             return;
-        fp value = ftk->get_variable(nr)->get_value();
-        for (int i = 0; i < ftk->get_dm_count(); ++i)
-            if (ftk->get_model(i)->get_ff_names().size() > 0)
-                cmd += "@" + S(i) + ".F." + p + " = " + S(value) + "; ";
-        // the variable was auto-deleted.
-        //cmd += "delete " + varname;
+        val_str = ftk->get_variable(nr)->get_formula(ftk->parameters());
+        // varname (_hwhm or _shape) will be auto-deleted
     }
+    string cmd;
+    for (int i = 0; i < ftk->get_dm_count(); ++i)
+        if (ftk->get_model(i)->get_ff().names.size() > 0) {
+            if (!cmd.empty())
+                cmd += "; ";
+            cmd += "@" + S(i) + ".F[*]." + p + " = " + val_str;
+        }
     ftk->exec(cmd);
 }
 
-void SideBar::on_parameter_changing(const std::vector<double>& values)
+void SideBar::on_parameter_changing(const std::vector<realt>& values)
 {
-    bool erase_previous = true;
-    frame->get_main_plot()->draw_xor_peak(pp_func, values, erase_previous);
+    frame->get_main_plot()->draw_overlay_func(pp_func, values);
 }
 
 void SideBar::on_parameter_changed(int n)
 {
-    frame->get_main_plot()->redraw_xor_peak(true); // erase xor line
     string vname = wx2s(param_panel->get_label2(n));
     ftk->exec(vname + " = ~" + eS(param_panel->get_value(n)));
 }
 
-void SideBar::on_parameter_lock_toggled(int n, bool locked)
+void SideBar::on_parameter_lock_clicked(int n, int state)
 {
     string vname = wx2s(param_panel->get_label2(n));
-    ftk->exec(vname + " = " + (locked ? "{" : "~{") + vname + "}");
+    if (state == 0)
+        ftk->exec(vname + " = ~{" + vname + "}");
+    else if (state == 1)
+        ftk->exec(vname + " = {" + vname + "}");
+    else { // state == 2
+        nb->SetSelection(2); // "variables" page
+        wxString vname_no_prefix = s2wx(vname.substr(1));
+        for (int i = 0; i != v->list->GetItemCount(); ++i) {
+            if (v->list->GetItemText(i) == vname_no_prefix) {
+                v->list->Select(i, true);
+                v->list->EnsureVisible(i);
+                v->list->Focus(i);
+            }
+            else
+                v->list->Select(i, false);
+        }
+    }
 }
 
 void SideBar::change_parameter_value(int idx, double value)
@@ -1062,7 +1095,7 @@ void SideBar::update_param_panel()
 
     pp_func = ftk->get_function(active_function);
 
-    wxString new_label = s2wx(pp_func->xname + " : " + pp_func->type_name);
+    wxString new_label = s2wx("%" + pp_func->name +" : "+ pp_func->tp()->name);
     if (param_panel->get_title() != new_label)
         param_panel->set_title(new_label);
 
@@ -1076,15 +1109,17 @@ void SideBar::update_param_panel()
         if (var->is_simple() || var->is_constant()) {
             bool locked = var->is_constant();
             param_panel->set_normal_parameter(i, label, var->get_value(),
-                                              locked, s2wx(var->xname));
+                                              locked, s2wx("$"+var->name));
         }
         else
             param_panel->set_disabled_parameter(i, label, var->get_value(),
-                                                s2wx(var->xname));
+                                                s2wx("$"+var->name));
     }
 
+    // Layout() is needed only when the layout has changed (e.g. when label2
+    // is shown for the first time). We call it always, just in case.
+    param_panel->Layout();
     if (new_count != old_count) {
-        param_panel->Layout();
         int sash_pos = GetClientSize().GetHeight() - 3
                          - param_panel->GetSizer()->GetMinSize().GetHeight();
         if (sash_pos < GetSashPosition())
